import java.lang.*;
import java.util.*;
import java.io.*;

import javax.swing.*;                                           
import java.awt.*;
import java.awt.event.*;                                                  
import javax.sound.midi.*;


class cellautorulec {
  public byte lookuptable[][];
  public Color cellcolor[];
  static public String cellnotename[] = {"F","C","G","D","A","E","B","F#","C#","G#","D#","A#"};
  static public int cellnote[] = new int[12];
  static public int cellnoteindex[] = new int[12];
  public int tablewidth = 16;
  public int tableheight = 2;
  public int numcolors = 8;
  public int rulechange = 2;
  public int seed = 0;
  public String description;
  public int neighborhoodsize = 5;
  cellautorulec(cellautorulec c) {
    tablewidth = c.tablewidth;
    tableheight = c.tableheight;
    neighborhoodsize = c.neighborhoodsize;
    lookuptable = new byte[tablewidth][tableheight];
    cellcolor = new Color[12];
    numcolors = c.numcolors;
    for (int i = 0;i < 12;i++) {
      cellcolor[i] = c.cellcolor[i];
      cellnote[i] = c.cellnote[i];
    }
    for (int x = 0;x < tablewidth;x++) {
      for (int y = 0;y < tableheight;y++) {
        lookuptable[x][y] = c.lookuptable[x][y];
      }
    }
  }
  
  cellautorulec() {
    lookuptable = new byte[tablewidth][tableheight];
    cellcolor = new Color[12];
    cellnote = new int[12];
    int i2 = 5;
    for (int i = 0;i < 12;i++) {
      cellnote[i] = i2;
      cellnoteindex[i2] = i;
      i2 = (i2 + 7) % 12;
    }
    numcolors = 8;
    cellcolor[0] = new Color(85,255,0);
    cellcolor[1] = new Color(255,0,0);
    cellcolor[2] = new Color(0,255,255);
    cellcolor[3] = new Color(255,128,0);
    cellcolor[4] = new Color(0,0,255);
    cellcolor[5] = new Color(255,255,0);
    cellcolor[6] = new Color(255,0,255);
    cellcolor[7] = new Color(0,255,145);
    cellcolor[8] = new Color(255,85,0);
    cellcolor[9] = new Color(0,128,255);
    cellcolor[10] = new Color(255,170,0);
    cellcolor[11] = new Color(128,0,255);
  }
  String get_cellnotename(int i) {
    int n = cellautoapp.key_to_play_in % 12;
    if (n < 0) {n = n + 12;}
    return cellnotename[(i-1+cellnoteindex[n]+12) % 12];
  }
  Color getcellvaluecolor(int i) {
    if (i < 0){System.out.println("note: cellvalue index is out of range");}
    if (i > 12){System.out.println("note: cellvalue index is out of range");}
    if (i == 0) {return Color.black;}
    if (cellcolor[i-1] == null){
      return Color.white;
    }else{
      int n = cellautoapp.key_to_play_in % 12;
      if (n < 0) {n = n + 12;}
      return cellcolor[(i-2+cellnoteindex[n]+12) % 12];
    }    
  }
  void setcellvaluecolor(int i,Color c) {
    if (i < 1){System.out.println("note: cellvalue index is out of range");}
    if (i > 12){System.out.println("note: cellvalue index is out of range");}
    cellcolor[i-1] = c;
  }  

  //this function writes to the rule table
  //Parameters:
  //  x: the entry's x offset or the neighborhood count
  //  y: the entry's y offset or the cell's state
  //  a: the entry's new value
  public void setlookuptable(int x,int y,int a) {
    if ((x < 0) | (y < 0) | (x >= tablewidth) | (y >= tableheight)) {
      System.out.println("note: table index is out of range" + x + " " + y);
      return;
    }
    if (a >= 128){ 
      a = a - 256;
    }
    lookuptable[x][y] = ((byte) a);
    rulechange = 2;
  }
  
  //this function reads from the rule table
  //Parameters:
  //  x: the entry's x offset or the neighborhood count
  //  y: the entry's y offset or the cell's state
  //Returns:
  //  the entry's value
  public int getlookuptable(int x,int y) {
    if ((x < 0) | (y < 0) | (x >= tablewidth) | (y >= tableheight)) {
    //  System.out.println("note: table index is out of range");
      return 0;
    }
    int a = lookuptable[x][y];
    if (a < 0) {a = a + 256;}
    return ((int) a);
  }
  
  public void doubletable() {
    int s = 1 << neighborhoodsize;
    int tmptable[] = new int[s];
    for (int i = 0;i < s;i++) {
      tmptable[i] = getlookuptablewithindex(i);
    }
    cleartable(neighborhoodsize + 1);
    for (int i = 0;i < s;i++) {
      setlookuptablewithindex((i*2)+0,tmptable[i]);
      setlookuptablewithindex((i*2)+1,tmptable[i]);
    }
  }
  public void cleartable(int s) {
    neighborhoodsize = s;
    int s2 = (neighborhoodsize + 3) >> 1;
    tablewidth = (1 << s2);
    tableheight = (1 << (neighborhoodsize-s2));
    lookuptable = new byte[tablewidth][tableheight];
  }
  public int getlookuptablewithindex(int i) {
    int s = (neighborhoodsize + 3) >> 1;
    return getlookuptable(i & ((1 << s) - 1),i >> s);
  }
  public void setlookuptablewithindex(int i,int a) {
    int s = (neighborhoodsize + 3) >> 1;
    setlookuptable(i & ((1 << s) - 1),i >> s,a);
  }

  public void calc(boardc b) {
    cellautoapp.board.generation = cellautoapp.board.generation + 1;
    int y = (cellautoapp.board.generation % cellautoapp.board.height);
    for (int x = 0;x < cellautoapp.board.width;x++) {
      int sq = b.getsq(x,y-1);
      int sq2 = 0;
      int sum = getsum(b,x,y);
      sq2 = this.getlookuptablewithindex(sum);
      b.setsq(x,y,sq2);
    }
  }

  int getsum(boardc b,int x,int y) {
    int sum = 0;
    int a2 = 0;
    for (int a = 0;a < neighborhoodsize;a++){
      sum = sum << 1;
      if (b.getsq(x+a2,y-1) > 0) {
        sum = sum | 1;
      }
      if (a2 >= 0) {a2 = a2 + 1;}
      a2 = -a2;
    }
    return sum;
  }
}  

class boardc {
  int sq[][];
  int width = 1;
  int height = 1;
  int generation = 0;

  int get_width(){
    return this.width;
  }
  int get_height(){
    return this.height;
  }

  boardc(int w,int h) {
    this.width = w;
    this.height = h;
    this.generation = 0;
    sq = new int[this.width][this.height];
  }
  boardc(boardc b) {
    this.width = b.get_width();
    this.height = b.get_height();
    this.generation = b.generation;
    sq = new int[this.width][this.height];
    this.copy(b);
  }
  void copy(boardc b){
    if (this.width != b.get_width()) {return;}
    if (this.height != b.get_height()) {return;}
    for (int x = 0;x < this.width;x++) {
      for (int y = 0;y < this.height;y++) {
        setsq(x,y,(byte) b.getsq(x,y));
      }
    }
  }

  //this function checks if a row is empty
  boolean emptyline(int r){
    boolean b = true;
    for (int i = 0;i < width;i++){
      int sq = getsq(i,r);
      if (sq != 0){
        b = false;
      }
    }
    return b;
  }  
    
  //this function reads a run length encoded string into a row
  void readrle(int r,String str){
    int l = str.length();
    int a0 = 0;
    int a1 = 0;
    int a2 = 0;
    int a3 = 0;
    
    if (r < 0){return;}
    if (r >= height){return;}
    for (int i = 0;i < l;i++){
      char ch = str.charAt(i);
      if ((ch >= 'a')& (ch <= 'z')){
        if (a0 == 0){
          for (int i2 = 0;i2 < a2;i2++){
            if (a3 < width){
              setsq(a3,r,a1);
              a3++;
            }
          }
          a1 = ch - 97;
          
          a0 = 1;
          a2 = 0;
        }else{
          a0 = 0;
          a1 = (a1 << 4) + (ch - 97);
          a2 = 0;
        }
      }
      if ((ch >= '0') & (ch <= '9')){
        a2 = (a2*10) + (ch - 48);
      }
    }
    for (int i2 = 0;i2 < a2;i2++){
      if (a3 < width){
        setsq(a3,r,a1);
        a3++;
      }
    }
    
  }
  
  //this function writes a row to a new run length encoded string
  String writerle(int r){
    StringBuffer str = new StringBuffer();
    int a0 = getsq(0,r);
    int a1 = getsq(0,r);
    int a2 = 0;
    for (int i = 0;i < width;i++){
      a1 = getsq(i,r);
      if (a0 != a1){
        char ch1 = (char) ((a0 >> 4) + 97);
        char ch2 = (char) ((a0 & 15) + 97);
        str.append(ch1);
        str.append(ch2);
        str.append(a2);
        a2 = 1;
      }else{
        a2++;
      }
      a0 = a1;
    }

    char ch1 = (char) ((a0 >> 4) + 97);
    char ch2 = (char) ((a0 & 15) + 97);
    str.append(ch1);
    str.append(ch2);
    str.append(a2);
    return str.toString();
  }

  static boardc new_random_board(Random rand,int width,int height,int offset,int length) {
    int numcolors = cellautoapp.cellauto131.numcolors;  
    boardc b = new boardc(width,height);
    b.generation = 0;           
    for (int x = 0;x < length;x++) {
      int a = rand.nextInt(2);
      if (a == 1) {
        int a2 = rand.nextInt(numcolors-1);
        b.setsq(x+offset,0,a2);
      } else {
        b.setsq(x+offset,0,0);
      }
    }
    return b;
  }
  void copy_rows(int x1,int y1,int x2,int y2) {
    for (int i = 0;i < width;i++) {
      int sq = getsq(i,y1);
      setsq(i,y2,sq);
    }
  }
  void swap_rows(int x1,int y1,int x2,int y2) {
    int dx = x2-x1;
    int tmp_row[] = new int[width];
    for (int i = 0;i < width;i++) {
      tmp_row[i] = getsq(i,y1);
      int sq = getsq(i,y2);
      setsq(i,y1,sq);
    }
    for (int i = 0;i < width;i++) {
      int sq = tmp_row[i];
      setsq(i+dx,y2,sq);
    }
  }
  int getsq(int x,int y){
    int x2 = 0;
    int y2 = 0;
    if ((this.width > 1) | (this.height > 1)){
      x2 = x % this.width;
      y2 = y % this.height;
      if (x2 < 0){x2 = x2 + this.width;}
      if (y2 < 0){y2 = y2 + this.height;}
    }
    return sq[x2][y2];
  }
  void setsq2(cellautorulec c,int x,int y,int sq4) {
    int sq = getsq(x,y-1);
    int sq2 = getsq(x,y);
    int sq3 = 0;
    int sum = c.getsum(this,x,y);
    
    
    sq3 = c.getlookuptablewithindex(sum);
    if ((sq3 == sq2) & (sq3 > 0) & (sq4 > 0)) {
      c.setlookuptablewithindex(sum,sq4);
    }    
    setsq(x,y,sq4);
    updateboard(c,sum,sq,sq2,sq3,sq4);
  }
  void updateboard(cellautorulec c,int sum,int sq,int sq2,int sq3,int sq4) {
    if ((sq3 == sq2) & (sq3 > 0) & (sq4 > 0) & (sq3 != sq4)) {
      for (int x2 = 0;x2 < this.width;x2++) {
        for (int y2 = 0;y2 < this.height;y2++) {
          if (sq3 == getsq(x2,y2)) {
            int sq5 = getsq(x2,y2-1);
            int sum2 = c.getsum(this,x2,y2);
            if (sum == sum2) {
              if ((sq5 > 0) & (sq > 0)) {
                setsq(x2,y2,sq4);
              }
              if ((sq5 == 0) & (sq == 0)) {
                setsq(x2,y2,sq4);
              }
            }
          }
        }
      }
    }
  }
  void updateboard2(cellautorulec c) {
      boardc b = this;
      for (int x2 = 0;x2 < b.width;x2++) {
        for (int y2 = 0;y2 < b.height;y2++) {
          if (b.getsq(x2,y2) > 0) {
            int sq = b.getsq(x2,y2-1);
            int sum = c.getsum(b,x2,y2);
            int sq2 = 0;
            sq2 = c.getlookuptablewithindex(sum);
            if (sq2 > 0) {
              b.setsq(x2,y2,sq2);
            }
          }          
        }
      }
  }
  void setsq(int x,int y,int a){
    int x2 = 0;
    int y2 = 0;
    if ((this.width > 1) | (this.height > 1)){
      x2 = x % this.width;
      y2 = y % this.height;
      if (x2 < 0){x2 = x2 + this.width;}
      if (y2 < 0){y2 = y2 + this.height;}
    }
    sq[x2][y2] = a;
  }
  


  int get_background_color(){
    return 0;
  }
  int get_background_color2(int x,int y,int t){
    return 0;
  }
  public boolean equals(boardc b){
    if (this.width == b.width) {return false;}
    if (this.height == b.height) {return false;}
    for (int x = 0;x < this.width;x++) {
      for (int y = 0;y < this.height;y++) {
        if (sq[x][y] != b.getsq(x,y)) {return false;}
      }
    }
    return true;
  }
}

class edittablemouselisc implements MouseListener{
  public void mouseClicked(MouseEvent e){
    
  }
  public void mouseEntered(MouseEvent e){
    
  } 
  public void mouseExited(MouseEvent e){
    
  }
  public void mousePressed(MouseEvent e){
    int x = e.getX() / edittablepanelc.sqsize;
    int y = e.getY() / edittablepanelc.sqsize;
    int x2 = cellautoapp.edittablewindow.edittablepanel.offsetx;
    int y2 = cellautoapp.edittablewindow.edittablepanel.offsety;
    if ((x+x2) < cellautoapp.cellauto131.tablewidth){
      if ((y+y2) < cellautoapp.cellauto131.tableheight){
        int i = cellautoapp.colorlistindex;
        int i2 = cellautoapp.cellauto131.getlookuptable(x+x2,y+y2);
        cellautoapp.cellauto131.setlookuptable(x+x2,y+y2,i);
        cellautoapp.board.updateboard(cellautoapp.cellauto131,x+x2,y+y2,i2,i2,i);
        cellautoapp.cellautomainwindow.repaint();
        cellautoapp.edittablewindow.edittablepanel.repaint();
      }
    }
  }
  public void mouseReleased(MouseEvent e){
    
  } 
}
class edittableadjlisc implements AdjustmentListener{
  public void adjustmentValueChanged(AdjustmentEvent e) {
    Adjustable a = e.getAdjustable();
    int i = e.getValue();
    if (a == cellautoapp.edittablewindow.hscroll){
      cellautoapp.edittablewindow.edittablepanel.offsetx = i;
    }
    if (a == cellautoapp.edittablewindow.vscroll){
      cellautoapp.edittablewindow.edittablepanel.offsety = i;
    }
    cellautoapp.edittablewindow.edittablepanel.repaint();
  }
}
class edittablewindowc extends JFrame {
  public edittablepanelc edittablepanel;
  public JScrollBar hscroll;
  public JScrollBar vscroll;
  edittableadjlisc adjlis;
  
  edittablewindowc(String Title){
    super(Title);
    adjlis = new edittableadjlisc();
    edittablepanel = new edittablepanelc();
    hscroll = new JScrollBar();
    vscroll = new JScrollBar();
    vscroll.setOrientation(JScrollBar.VERTICAL);
    vscroll.setMinimum(0);
    vscroll.setVisibleAmount(0);
    vscroll.setMaximum(cellautoapp.cellauto131.tableheight);
    vscroll.addAdjustmentListener(adjlis);
    hscroll.setMinimum(0);
    hscroll.setVisibleAmount(0);
    hscroll.setMaximum(cellautoapp.cellauto131.tablewidth);
    hscroll.addAdjustmentListener(adjlis);
    hscroll.setOrientation(JScrollBar.HORIZONTAL);
    SpringLayout spl = new SpringLayout();
    this.getContentPane().setLayout(spl);
    spl.putConstraint(SpringLayout.EAST,vscroll,0,SpringLayout.EAST,this.getContentPane());    
    spl.putConstraint(SpringLayout.EAST,edittablepanel,0,SpringLayout.WEST,vscroll);
    spl.putConstraint(SpringLayout.WEST,edittablepanel,0,SpringLayout.WEST,this.getContentPane());
    
    spl.putConstraint(SpringLayout.SOUTH,hscroll,0,SpringLayout.SOUTH,this.getContentPane());    
    spl.putConstraint(SpringLayout.SOUTH,edittablepanel,0,SpringLayout.NORTH,hscroll);    
    spl.putConstraint(SpringLayout.NORTH,edittablepanel,0,SpringLayout.NORTH,this.getContentPane());
    
    spl.putConstraint(SpringLayout.SOUTH,vscroll,0,SpringLayout.NORTH,hscroll);    
    spl.putConstraint(SpringLayout.NORTH,vscroll,0,SpringLayout.NORTH,this.getContentPane());    
    
    spl.putConstraint(SpringLayout.EAST,hscroll,0,SpringLayout.WEST,vscroll);    
    spl.putConstraint(SpringLayout.WEST,hscroll,0,SpringLayout.WEST,this.getContentPane());    
    this.getContentPane().add(edittablepanel);
    this.getContentPane().add(vscroll);
    this.getContentPane().add(hscroll);
    this.pack();  
    edittablepanel.setFocusable(true);
  }
};

class edittablepanelc extends JPanel{
  int offsetx;
  int offsety;
  edittablemouselisc mouselis;
  static int sqsize = 16;
  static int grid_x_spacing = 4;
  static int grid_y_spacing = 4;
  static int grid_x_spacing2 = 8;
  static int grid_y_spacing2 = 8;
  static Color gridcolor1 = Color.blue;
  static Color gridcolor2 = Color.cyan;
  static Color gridcolor3 = Color.white;
  edittablepanelc(){
    mouselis = new edittablemouselisc();
    this.addMouseListener(mouselis);
    
    this.addKeyListener(cellautoapp.keylis);
    
    offsetx = 0;
    offsety = 0;
  }
  //this function draws the rule table
  protected void paintComponent(Graphics g){ 
    super.paintComponent(g);
    int h = (getHeight() / sqsize) + 1;
    int w = (getWidth() / sqsize) + 1;
    
    //draw the rule table
    for (int y = 0;y < h;y++){
      for (int x = 0;x < w;x++){
        if ((x+offsetx) < cellautoapp.cellauto131.tablewidth){
          if ((y+offsety) < cellautoapp.cellauto131.tableheight){
            int i = cellautoapp.cellauto131.getlookuptable(x+offsetx,y+offsety);
            g.setColor(cellautoapp.cellauto131.getcellvaluecolor(i));
            g.fillRect(x*sqsize,y*sqsize,sqsize,sqsize);
          }
        }
      }
    }
    int w2 = w;
    int h2 = h;
    if (w2 >= (cellautoapp.cellauto131.tablewidth-offsetx)){
      w2 = cellautoapp.cellauto131.tablewidth-offsetx;
    }
    if (h2 >= (cellautoapp.cellauto131.tableheight-offsety)){
      h2 = cellautoapp.cellauto131.tableheight-offsety;
    }
    
    //draw the grid
    for (int x = 1;x <= w2;x++){
      g.setColor(gridcolor1);
      if (((x+offsetx) % grid_x_spacing) == 0){g.setColor(gridcolor2);}
      if (((x+offsetx) % grid_x_spacing2) == 0){g.setColor(gridcolor3);}
      g.drawLine(x*sqsize,0,x*sqsize,h2*sqsize);
    }
    for (int y = 1;y <= h2;y++){
      g.setColor(gridcolor1);
      if (((y+offsety) % grid_y_spacing) == 0){g.setColor(gridcolor2);}
      if (((y+offsety) % grid_y_spacing2) == 0){g.setColor(gridcolor3);}
      g.drawLine(0,y*sqsize,w2*sqsize,y*sqsize);
    }
    
  }  
};


class newgamedialogc extends JFrame {
  JLabel boardlabel;
  JLabel widthlabel;
  JTextField widthedit;
  JLabel heightlabel;
  JTextField heightedit;

  JLabel linelabel;
  JLabel seedlabel;
  JTextField seededit;
  JLabel offsetlabel;
  JTextField offsetedit;
  JLabel lengthlabel;
  JTextField lengthedit;

  JButton okbutton;
  JButton applybutton;
  JButton cancelbutton;
  JButton clearbutton;

  //int seed;
  newgamedialogc(String Title) {
    super(Title);
    //seed = 1;
    
    GridBagLayout gl = new GridBagLayout();
    this.getContentPane().setLayout(gl);
    GridBagConstraints c = new GridBagConstraints();
    
    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1.0;
    c.weighty = 1.0;
    
    c.gridwidth = GridBagConstraints.REMAINDER;
    boardlabel = new JLabel();
    boardlabel.setText("board size");
    gl.setConstraints(boardlabel, c);
    
    c.gridwidth = 3;
    widthlabel = new JLabel();
    widthlabel.setText("width");
    gl.setConstraints(widthlabel, c);

    widthedit = new JTextField();
    widthedit.setText(String.valueOf(cellautoapp.board.width));
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(widthedit, c);

    heightlabel = new JLabel();
    heightlabel.setText("height");
    c.gridwidth = 3;
    gl.setConstraints(heightlabel, c);

    heightedit = new JTextField();
    heightedit.setText(String.valueOf(cellautoapp.board.height));
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(heightedit, c);

    c.gridwidth = GridBagConstraints.REMAINDER;
    linelabel = new JLabel();
    linelabel.setText("line of random cells");
    gl.setConstraints(linelabel, c);

    c.gridwidth = 3;
    seedlabel = new JLabel();
    seedlabel.setText("seed");
    gl.setConstraints(seedlabel, c);
    
    c.gridwidth = GridBagConstraints.REMAINDER;
    seededit = new JTextField();
    seededit.setText(Long.toString(cellautoapp.board_song_seed));
    gl.setConstraints(seededit, c);
    
    c.gridwidth = 3;
    offsetlabel = new JLabel();
    offsetlabel.setText("offset");
    gl.setConstraints(offsetlabel, c);
    
    c.gridwidth = GridBagConstraints.REMAINDER;
    offsetedit = new JTextField();
    offsetedit.setText("0");
    gl.setConstraints(offsetedit, c);

    c.gridwidth = 3;
    lengthlabel = new JLabel();
    lengthlabel.setText("length");
    gl.setConstraints(lengthlabel, c);
    
    c.gridwidth = GridBagConstraints.REMAINDER;
    lengthedit = new JTextField();
    lengthedit.setText("100");
    gl.setConstraints(lengthedit, c);

    okbutton = new JButton();
    okbutton.setText("ok");
    okbutton.setActionCommand("newgameokbuttonclick");
    okbutton.addActionListener(cellautoapp.actlis);
    c.gridwidth = 2;
    
    gl.setConstraints(okbutton, c);
    
    applybutton = new JButton();
    applybutton.setText("apply");
    applybutton.setActionCommand("newgameapplybuttonclick");
    applybutton.addActionListener(cellautoapp.actlis);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(applybutton, c);
    
    cancelbutton = new JButton();
    cancelbutton.setText("cancel");
    cancelbutton.setActionCommand("newgamecancelbuttonclick");
    cancelbutton.addActionListener(cellautoapp.actlis);
    c.gridwidth = 2;
    gl.setConstraints(cancelbutton, c);
    
    
    clearbutton = new JButton();
    clearbutton.setText("clear");
    clearbutton.setActionCommand("newgameclearbuttonclick");
    clearbutton.addActionListener(cellautoapp.actlis);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(clearbutton, c);

    this.getContentPane().add(boardlabel);
    this.getContentPane().add(widthlabel);
    this.getContentPane().add(widthedit);
    this.getContentPane().add(heightlabel);
    this.getContentPane().add(heightedit);
    this.getContentPane().add(linelabel);
    this.getContentPane().add(seedlabel);
    this.getContentPane().add(seededit);
    this.getContentPane().add(offsetlabel);
    this.getContentPane().add(offsetedit);
    this.getContentPane().add(lengthlabel);
    this.getContentPane().add(lengthedit);
    this.getContentPane().add(okbutton);
    this.getContentPane().add(applybutton);
    this.getContentPane().add(cancelbutton);
    this.getContentPane().add(clearbutton);
    this.pack();
  }
  void clear() {
    int width = Integer.parseInt(widthedit.getText().trim());
    int height = Integer.parseInt(heightedit.getText().trim());
    cellautoapp.board = new boardc(width,height);
    cellautoapp.board.generation = 0;
    cellautoapp.updatestatusbar();
    cellautoapp.cellautomainwindow.repaint();
  }

  void newgame() {
    int numcolors = cellautoapp.cellauto131.numcolors;  
    int width = Integer.parseInt(widthedit.getText().trim());
    int height = Integer.parseInt(heightedit.getText().trim());
    long seed2 = Long.parseLong(seededit.getText().trim());            
    int offset = Integer.parseInt(offsetedit.getText().trim());            
    int length = Integer.parseInt(lengthedit.getText().trim());            

    
    cellautoapp.board_song_seed = seed2;
    cellautoapp.update_board_song_seed();
    seededit.setText(Long.toString(cellautoapp.board_song_seed));

    cellautoapp.rand.setSeed(cellautoapp.board_song_seed);
    cellautoapp.board = boardc.new_random_board(cellautoapp.rand,width,height,offset,length);
    cellautoapp.updatestatusbar();
    cellautoapp.cellautomainwindow.repaint();
    int w2 = cellautoapp.cellautomainwindow.getWidth() / cellautomainwindowc.sqsize;
    int h2 = cellautoapp.cellautomainwindow.getHeight() / cellautomainwindowc.sqsize;

    cellautoapp.offsetx = offset-((w2-length)/2);
    cellautoapp.offsety = 0;
  }
}

class winlisc implements WindowListener{
  public void windowActivated(WindowEvent e){
    cellautoapp.cellautowindowframe.requestFocusInWindow();
    cellautoapp.cellautomainwindow.repaint();
  }
  public  void windowClosed(WindowEvent e){
  
  }
  public void windowClosing(WindowEvent e){
    //System.out.println("exiting");
    try{
      System.exit(0);
    }catch(SecurityException err){
      System.out.println("can not exit");
    }
  }
  public void windowDeactivated(WindowEvent e){
  }
  public void windowDeiconified(WindowEvent e){
  }
  public void windowIconified(WindowEvent e){
  }
  public void windowOpened(WindowEvent e){
  }
}
class cellautomainwindowc extends JPanel{
  static int sqsize = 20;
  cellautomainwindowc() {
  }
  void paintsquare(Graphics g,int x,int y,int a){
    int a2 = 0;
    if ((cellautoapp.gridon) & (sqsize >= cellautoapp.gridminsize)){a2 = 1;}
    
    g.setColor(cellautoapp.cellauto131.getcellvaluecolor(a));
    g.fillRect((x*sqsize)+a2,(y*sqsize)+a2,sqsize-a2,sqsize-a2);
  }
 
  protected void paintComponent(Graphics g){ 
    super.paintComponent(g);
    Color b = new Color(0,0,0);
    Color w = new Color(255,255,255);
    int w2 = (this.getWidth() / sqsize) + 1;
    int h2 = (this.getHeight() / sqsize)+ 1;
    int w3 = cellautoapp.board.width;
    int h3 = cellautoapp.board.height;    
    g.setColor(Color.black);
    g.fillRect(0,0,this.getWidth(),this.getHeight());
    for (int x = 0;x < w2;x++){
      for (int y = 0;y < h2;y++){
        int x2 = x + cellautoapp.offsetx;
        int y2 = y + cellautoapp.offsety;
        if ((x2 >= 0) & (x2 < cellautoapp.board.width)) {
          if ((y2 >= 0) & (y2 < cellautoapp.board.height)) {
            int a = cellautoapp.board.getsq(x2,y2);
            if (a != 0) {
              paintsquare(g,x,y,a);
            }
          }
        }
      }
    }

    
      
    int x1 = 0;
    int x2 = w2;
    int y1 = 0;
    int y2 = h2;
    if (-cellautoapp.offsetx >= x1) {x1 = -cellautoapp.offsetx;}
    if ((w3 - cellautoapp.offsetx) <= x2) {x2 = w3 - cellautoapp.offsetx;}
    if (-cellautoapp.offsety >= y1) {y1 = -cellautoapp.offsety;}
    if ((h3 - cellautoapp.offsety) <= y2) {y2 = h3 - cellautoapp.offsety;}
    
    if ((x1 < x2) & (y1 < y2)) {
      if (sqsize > cellautoapp.gridminsize) {
        for (int x = x1;x <= x2;x++){
          if (((x + cellautoapp.offsetx) % 16) == 0) {g.setColor(Color.green);}
          else if (((x + cellautoapp.offsetx) % 4) == 0) {g.setColor(Color.cyan);}
          else {g.setColor(Color.blue);}
          g.drawLine(x*sqsize,y1*sqsize,x*sqsize,y2*sqsize);
        
        }
        for (int y = y1;y <= y2;y++){
          if (((y + cellautoapp.offsety) % 16) == 0) {g.setColor(Color.green);}
          else if (((y + cellautoapp.offsety) % 4) == 0) {g.setColor(Color.cyan);}
          else {g.setColor(Color.blue);}
          g.drawLine(x1*sqsize,y*sqsize,x2*sqsize,y*sqsize);
        }
      }
      //draw board edges
      g.setColor(Color.red);
      if (x1 == -cellautoapp.offsetx) {
        g.drawLine(x1*sqsize,y1*sqsize,x1*sqsize,y2*sqsize);
      }
      if (x2 == (w3 - cellautoapp.offsetx)) {
        g.drawLine(x2*sqsize,y1*sqsize,x2*sqsize,y2*sqsize);
      }
      if (y1 == -cellautoapp.offsety) {
        g.drawLine(x1*sqsize,y1*sqsize,x2*sqsize,y1*sqsize);
      }
      if (y2 == (h3 - cellautoapp.offsety)) {
        g.drawLine(x1*sqsize,y2*sqsize,x2*sqsize,y2*sqsize);
      }
      int line_pos = (cellautoapp.board.generation % cellautoapp.board.height) - cellautoapp.offsety + 1;
      if ((line_pos >= 0) & (line_pos <= h2)) {
        g.setColor(Color.white);
        if (sqsize >= 2) {
          g.drawLine(x1*sqsize,line_pos*sqsize,x2*sqsize,line_pos*sqsize);
        }
        if (sqsize >= 6) {
          g.drawLine(x1*sqsize,(line_pos*sqsize)+1,x2*sqsize,(line_pos*sqsize)+1);
        }
      }
    }

    
  }
}
class mousemotionlisc implements MouseMotionListener{
  int getsqx(int x){
    x = x - 0;
    x = x / cellautomainwindowc.sqsize;
    x = x + cellautoapp.offsetx;
    return x;
  }
  int getsqy(int y){
    y = y - 0;
    y = y / cellautomainwindowc.sqsize;
    y = y + cellautoapp.offsety;
    return y;
  }
  public void mouseDragged(MouseEvent e){
    Graphics g = cellautoapp.cellautomainwindow.getGraphics();
    int x = getsqx(e.getX());
    int y = getsqy(e.getY());
    if ((cellautoapp.edit_rows_mode == false) & (cellautoapp.edit_notes_mode == false)) {
      if (cellautoapp.auto_update_rule_table == false) {
        cellautoapp.board.setsq2(cellautoapp.cellauto131,x,y,cellautoapp.colorlistindex);
      } else {
        cellautoapp.board.setsq(x,y,cellautoapp.colorlistindex);
      } 
      cellautoapp.cellautomainwindow.paintsquare(g,x-cellautoapp.offsetx,y-cellautoapp.offsety,cellautoapp.colorlistindex);
    }
  }
  public void mouseMoved(MouseEvent e){
    
  }
  
}

class mouselisc implements MouseListener{
  int x1;
  int y1;
  int x2;
  int y2;
  
  
  int getsqx(int x){
    x = x - 0;
    x = x / cellautomainwindowc.sqsize;
    x = x + cellautoapp.offsetx;
    return x;
  }
  int getsqy(int y){
    y = y - 0;
    y = y / cellautomainwindowc.sqsize;
    y = y + cellautoapp.offsety;
    return y;
  }
  public void mouseClicked(MouseEvent e){
    //System.out.println("clicked");
  }
  public void mouseEntered(MouseEvent e){
    //System.out.println("entered");
    
  }
  public void mouseExited(MouseEvent e){
    
    //System.out.println("exited");
  }
  public void mousePressed(MouseEvent e){
    //  System.out.println("pressed");
    x1 = getsqx(e.getX());
    y1 = getsqy(e.getY());
    
    
  }
  public void mouseReleased(MouseEvent e){
    x2 = getsqx(e.getX());
    y2 = getsqy(e.getY());
    if (cellautoapp.edit_rows_mode == true) {
      if (e.isControlDown()){
        cellautoapp.board.copy_rows(x1,y1,x2,y2);
      } else {
        cellautoapp.board.swap_rows(x1,y1,x2,y2);
      }
    } else {
      if (cellautoapp.edit_notes_mode == true) {
        cellautoapp.move_notes_around(x1,y1,x2,y2);
      } else {
          if (cellautoapp.auto_update_rule_table == true) {
            cellautoapp.board.setsq2(cellautoapp.cellauto131,x2,y2,cellautoapp.colorlistindex);
          } else {
            cellautoapp.board.setsq(x2,y2,cellautoapp.colorlistindex);
          }
      }
    }
    cellautoapp.cellautomainwindow.repaint();
    cellautoapp.edittablewindow.repaint();
  }
}

//note: 
//these keys below are just the defaults
//the keys usually are read from options.txt
//to configure the keys, edit options.txt
class cellautokeys {

  static String options_ctrl_key = "P";
  static String open_ctrl_key = "O";
  static String save_ctrl_key = "S";
  static String random_table_ctrl_key = "R";
  static String random_notes_ctrl_key = "T";

  static char edit_rows_mode_key = 'r';
  static char edit_notes_mode_key = 'e';

  static char prev_color_key = 'q';
  static char next_color_key = 'w';
  static char play_row_key = 's';
  
  static char save_to_stack_key = 'x';       
  static char undo_key = 'z';
  static char redo_key = 'a';

  static char no_note_key = 'g';
  static char C_key = 'c';
  static char CS_key = 'd';
  static char D_key = 'v';
  static char DS_key = 'f';
  static char E_key = 'b';
  static char F_key = 'n';
  static char FS_key = 'h';
  static char G_key = 'm';
  static char GS_key = 'j';
  static char A_key = ',';
  static char AS_key = 'k';
  static char B_key = '.';
  
}

class colorboxadjlisc implements AdjustmentListener{
  public void adjustmentValueChanged(AdjustmentEvent e) {
    int i = e.getValue();
    cellautoapp.colorbox.colorboxpanel.offset = i;
    
    cellautoapp.colorbox.colorboxpanel.repaint();
    
  }
}

class editdescriptionwindowc extends JFrame {
  public JScrollPane text2; 
  public JTextArea text; 
  public JButton okbutton;
  public JButton cancelbutton;
  editdescriptionwindowc(String Title){
    super(Title);
    SpringLayout spl = new SpringLayout();
    this.getContentPane().setLayout(spl);
    text = new JTextArea();
    text2 = new JScrollPane(text);
    okbutton = new JButton();
    okbutton.setText("ok");
    cancelbutton = new JButton();
    cancelbutton.setText("cancel");
    spl.putConstraint(SpringLayout.SOUTH,okbutton,-5,SpringLayout.SOUTH,this.getContentPane());
    spl.putConstraint(SpringLayout.SOUTH,cancelbutton,-5,SpringLayout.SOUTH,this.getContentPane());
    spl.putConstraint(SpringLayout.SOUTH,text2,-5,SpringLayout.NORTH,okbutton);
    spl.putConstraint(SpringLayout.WEST,cancelbutton,5,SpringLayout.EAST,okbutton);
    spl.putConstraint(SpringLayout.WEST,okbutton,5,SpringLayout.WEST,this.getContentPane());
    spl.putConstraint(SpringLayout.NORTH,text2,5,SpringLayout.NORTH,this.getContentPane());
    spl.putConstraint(SpringLayout.EAST,text2,-5,SpringLayout.EAST,this.getContentPane());
    spl.putConstraint(SpringLayout.WEST,text2,5,SpringLayout.WEST,this.getContentPane());

    okbutton.setActionCommand("descriptionokbuttonclick");
    okbutton.addActionListener(cellautoapp.actlis);
    cancelbutton.setActionCommand("descriptioncancelbuttonclick");
    cancelbutton.addActionListener(cellautoapp.actlis);
    text.setText(cellautoapp.cellauto131.description);
    this.getContentPane().add(text2);
    this.getContentPane().add(okbutton);
    this.getContentPane().add(cancelbutton);
    this.pack();  

    
  }
  void okbuttonclick(){
    cellautoapp.cellauto131.description = text.getText();
    setVisible(false);
  }
  void cancelbuttonclick(){
    setVisible(false);
  }
}

class actlisc implements ActionListener {   
  public void actionPerformed(ActionEvent e) {
    String action = e.getActionCommand();
    System.out.println(action);
    if (action.equals("menudescription")) {
      if (cellautoapp.editdescriptionwindow == null) {
        cellautoapp.editdescriptionwindow = new editdescriptionwindowc("edit description");
        cellautoapp.editdescriptionwindow.setBounds(0,0,200,200);
      }
      cellautoapp.editdescriptionwindow.text.setText(cellautoapp.cellauto131.description);
      cellautoapp.editdescriptionwindow.setVisible(true);
    }
    if (action.equals("descriptionokbuttonclick")) {
      cellautoapp.editdescriptionwindow.okbuttonclick();
    }
    if (action.equals("descriptioncancelbuttonclick")) {
      cellautoapp.editdescriptionwindow.cancelbuttonclick();
    }
    if (action.equals("showrandomtabledialog")) {
      if (cellautoapp.randomtabledialog == null) {
        cellautoapp.randomtabledialog = new randomtabledialogc("random table window");
        cellautoapp.randomtabledialog.setBounds(0,0,400,300);
      }
      cellautoapp.randomtabledialog.showoptions();
      cellautoapp.randomtabledialog.setVisible(true);
    }
    if (action.equals("menushowedittablewindow")) {
      cellautoapp.edittablewindow.setVisible(true);
    }
    if (action.equals("newgameokbuttonclick")) {
      cellautoapp.newgamedialog.newgame();
      cellautoapp.newgamedialog.setVisible(false);
    }
    if (action.equals("newgameapplybuttonclick")) {
      cellautoapp.newgamedialog.newgame();
    }
    if (action.equals("newgameclearbuttonclick")) {
      cellautoapp.newgamedialog.clear();
    }
    if (action.equals("newgamecancelbuttonclick")) {
      cellautoapp.newgamedialog.setVisible(false);
    }
    if (action.equals("menudoubletable")) {
      int r = JOptionPane.showConfirmDialog(cellautoapp.cellautowindowframe,"double the rule table size?","question",JOptionPane.YES_NO_OPTION);
      if (r == JOptionPane.YES_OPTION) {                                   
        cellautoapp.cellauto131.doubletable();
        cellautoapp.edittablewindow.repaint();        
      }
    }
    if (action.equals("menucleartable")) {
      String str2 = Integer.toString(cellautoapp.cellauto131.neighborhoodsize);
      String str = JOptionPane.showInputDialog("what should the new neighborhood width or size be?",str2);
      if (str != null) {
        int i = Integer.valueOf(str.trim()).intValue();
        cellautoapp.cellauto131.cleartable(i);
        cellautoapp.edittablewindow.repaint();        
      }
    }
    if (action.equals("menushowedittablewindow")) {
      cellautoapp.edittablewindow.setVisible(true);
    }
    if (action.equals("menushowcolorbox")) {
      cellautoapp.colorbox.setVisible(true);
    }
    if (action.equals("menurandomtable")) {


      cellautoapp.randomtable2();
    }
    if (action.equals("showplaylist")) {
      if (cellautoapp.playlistwindow == null) {
        cellautoapp.playlistwindow = new playlistwindowc("play list");
        cellautoapp.playlistwindow.setBounds(0,0,200,400);
      }
      cellautoapp.playlistwindow.showoptions();
      cellautoapp.playlistwindow.setVisible(true);
    }
    if (action.equals("menurandomnotes")) {
      cellautoapp.randomnotes2();
    }
    if (action.equals("menuaddnote")) {
      if (cellautoapp.cellauto131.numcolors <= 12) {
        cellautoapp.cellauto131.numcolors = cellautoapp.cellauto131.numcolors + 1;
      }
      cellautoapp.colorbox.colorboxpanel.repaint();

    }
    if (action.equals("menunew")) {
      if (cellautoapp.newgamedialog == null) {
        cellautoapp.newgamedialog = new newgamedialogc("new song");
      }
      //cellautoapp.newgamedialog.setText(Long.toString(cellautoapp.board_song_seed));
      cellautoapp.newgamedialog.setVisible(true);
    }

    if (action.equals("menuopen")) {
      try{
        FileDialog f = new FileDialog(cellautoapp.cellautowindowframe,"open",FileDialog.LOAD);
        f.show();
        if (f.getFile() != null) {
          //cellautoapp.cellauto131.filename = f.getFile();
          String filename = f.getDirectory() + f.getFile();
          cellautoapp.open(filename);
        }
        cellautoapp.cellautowindowframe.requestFocusInWindow();
        cellautoapp.cellautomainwindow.repaint();
      } catch (Exception err) {
        System.out.println("menuopen error");
        System.out.println(err.getMessage());
        err.printStackTrace();
      }
    }

    if (action.equals("menusave")) {
      try{
        FileDialog f = new FileDialog(cellautoapp.cellautowindowframe,"save",FileDialog.SAVE);
        f.show();
        if (f.getFile() != null) {
          //cellautoapp.cellauto131.filename = f.getFile();
          String filename = f.getDirectory() + f.getFile();
          cellautoapp.save(filename);
          
          //cellautoapp.cellautomainwindow.setFocusTraversalKeysEnabled(true);
          cellautoapp.cellautomainwindow.repaint();
        }
        cellautoapp.cellautowindowframe.requestFocusInWindow();
      } catch (Exception err) {
        System.out.println("menusave error");
        System.out.println(err.getMessage());
        err.printStackTrace();
      }
    }
 
    if (action.equals("menusaveasxm")) {
      try{
        FileDialog f = new FileDialog(cellautoapp.cellautowindowframe,"save as xm",FileDialog.SAVE);
        f.show();
        if (f.getFile() != null) {
          //cellautoapp.cellauto131.filename = f.getFile();
          String filename = f.getDirectory() + f.getFile();
          cellautoapp.saveasxm(filename);
          
          //cellautoapp.cellautomainwindow.setFocusTraversalKeysEnabled(true);
          cellautoapp.cellautomainwindow.repaint();
        }
        cellautoapp.cellautowindowframe.requestFocusInWindow();
      } catch (Exception err) {
        System.out.println("menusave error");
        System.out.println(err.getMessage());
        err.printStackTrace();
      }      
    }
    
    
    if (action.equals("menusaveasmidi")) {
      try{
        FileDialog f = new FileDialog(cellautoapp.cellautowindowframe,"save as midi",FileDialog.SAVE);
        f.show();
        if (f.getFile() != null) {
          //cellautoapp.cellauto131.filename = f.getFile();
          String filename = f.getDirectory() + f.getFile();
          cellautoapp.saveasmidi(filename);
          
          //cellautoapp.cellautomainwindow.setFocusTraversalKeysEnabled(true);
          cellautoapp.cellautomainwindow.repaint();
        }
        cellautoapp.cellautowindowframe.requestFocusInWindow();
      } catch (Exception err) {
        System.out.println("menusave error");
        System.out.println(err.getMessage());
        err.printStackTrace();
      }
    }
    
    if (action.equals("menuoptions")) {
      if (cellautoapp.optionsdialog == null) {
        cellautoapp.optionsdialog = new optionsdialogc("options");
        cellautoapp.optionsdialog.setBounds(0,0,400,400);
      }
      cellautoapp.optionsdialog.getoptions();
      cellautoapp.optionsdialog.setVisible(true);
    }
    if (action.equals("menuexit")) {
      try {
        System.exit(0);
      } catch(SecurityException err) {
        System.out.println("can not exit");
      }
    }
    
  }
}
class keylisc implements KeyListener{
  public void keyPressed(KeyEvent e){
    int a = e.getKeyCode();
    char ch = e.getKeyChar();  
    String s = e.getKeyText(a);
    ch = Character.toLowerCase(ch);
    if (e.isControlDown()){
      if (s.equals(cellautokeys.open_ctrl_key)) {
        try{
          FileDialog f = new FileDialog(cellautoapp.cellautowindowframe,"game open",FileDialog.LOAD);
          f.show();
          if (f.getFile() != null){
            //cellautoapp.cellauto131.filename = f.getFile();
            String filename = f.getDirectory() + f.getFile();
            cellautoapp.open(filename);
          }
          cellautoapp.cellautomainwindow.repaint();
          cellautoapp.cellautowindowframe.requestFocusInWindow();
          cellautoapp.cellautomainwindow.repaint();
          
        } catch (Exception err){
          System.out.println("menuopen error");
          System.out.println(err.getMessage());
          err.printStackTrace();
        }
      }
      if (s.equals(cellautokeys.save_ctrl_key)){
        try{
          FileDialog f = new FileDialog(cellautoapp.cellautowindowframe,"game save",FileDialog.SAVE);
          f.show();
          if (f.getFile() != null){
            //cellautoapp.cellauto131.filename = f.getFile();
            String filename = f.getDirectory() + f.getFile();
            cellautoapp.save(filename);
          }
          cellautoapp.cellautomainwindow.repaint();
          cellautoapp.cellautowindowframe.requestFocusInWindow();
          cellautoapp.cellautomainwindow.repaint();
          
        } catch (Exception err){
          System.out.println("menusave error");
          System.out.println(err.getMessage());
          err.printStackTrace();
        }
      }
      if (s.equals(cellautokeys.options_ctrl_key)){
        if (cellautoapp.optionsdialog == null) {
          cellautoapp.optionsdialog = new optionsdialogc("options");
          cellautoapp.optionsdialog.setBounds(0,0,400,200);
        }
        cellautoapp.optionsdialog.getoptions();
        cellautoapp.optionsdialog.setVisible(true);
      }
      if (s.equals(cellautokeys.random_table_ctrl_key)) {
        cellautoapp.randomtable2();
      }
      if (s.equals(cellautokeys.random_notes_ctrl_key)) {
        cellautoapp.randomnotes2();
        
      }
    }
    if ((ch == '+')|(ch == '=')){
      int x = cellautoapp.offsetx + (cellautoapp.cellautomainwindow.getWidth()/(cellautomainwindowc.sqsize*2));
      int y = cellautoapp.offsety + (cellautoapp.cellautomainwindow.getHeight()/(cellautomainwindowc.sqsize*2));

      cellautoapp.zoom(x,y,cellautoapp.ZOOM_IN);
      //cellautoapp.cellautomainwindow.repaint();
    }
    if (ch == cellautokeys.play_row_key) {
      cellautoapp.playlistrunning = false;
      cellautoapp.CA_Player.clearnotelist();
      for (int i = 0;i < cellautoapp.num_rows_to_play;i++){
        int i2 = (cellautoapp.board.generation+i-cellautoapp.num_rows_to_play+1) % cellautoapp.board.height;
        cellautoapp.CA_Player.play_row(cellautoapp.board,i2);
      }
      cellautoapp.CA_Player.play_last_note();
      notelistc l = cellautoapp.CA_Player.getnotelist();
      l.addchordstonotes();
      cellautoapp.note_player.playnotelist(l,cellautoapp.tempo);
    }
    if (ch == cellautokeys.no_note_key) {
      cellautoapp.colorlistindex = 0;
      cellautoapp.colorbox.colorboxpanel.selcolor = 0;
      cellautoapp.colorbox.colorboxpanel.repaint();
      cellautoapp.updatestatusbar();    
    }
    if (ch == cellautokeys.C_key) {
      cellautoapp.playnoteonkeyboard(60);
    }                                
    if (ch == cellautokeys.CS_key) { 
      cellautoapp.playnoteonkeyboard(61);
    }                                
    if (ch == cellautokeys.D_key) {  
      cellautoapp.playnoteonkeyboard(62);
    }                                
    if (ch == cellautokeys.DS_key) { 
      cellautoapp.playnoteonkeyboard(63);
    }                                
    if (ch == cellautokeys.E_key) {  
      cellautoapp.playnoteonkeyboard(64);
    }                                
    if (ch == cellautokeys.F_key) {  
      cellautoapp.playnoteonkeyboard(65);
    }                                
    if (ch == cellautokeys.FS_key) { 
      cellautoapp.playnoteonkeyboard(66);
    }                                
    if (ch == cellautokeys.G_key) {  
      cellautoapp.playnoteonkeyboard(67);
    }                                
    if (ch == cellautokeys.GS_key) { 
      cellautoapp.playnoteonkeyboard(68);
    }                                
    if (ch == cellautokeys.A_key) {  
      cellautoapp.playnoteonkeyboard(69);
    }                                
    if (ch == cellautokeys.AS_key) { 
      cellautoapp.playnoteonkeyboard(70);
    }                                
    if (ch == cellautokeys.B_key) {  
      cellautoapp.playnoteonkeyboard(71);
    }
    if (ch == cellautokeys.redo_key){
      if (cellautoapp.redodata.empty() == false) {
        cellautoapp.saveboard(cellautoapp.undodata);
        cellautoapp.restoreboard(cellautoapp.redodata);
      }
      cellautoapp.updatestatusbar();
      cellautoapp.cellautomainwindow.repaint();
      cellautoapp.edittablewindow.edittablepanel.repaint();
      cellautoapp.colorbox.colorboxpanel.repaint();
      
    }
    
    if (ch == cellautokeys.undo_key) {
      if (cellautoapp.undodata.empty() == false) {
        cellautoapp.saveboard(cellautoapp.redodata);
        cellautoapp.restoreboard(cellautoapp.undodata);
      }
      cellautoapp.updatestatusbar();
      cellautoapp.cellautomainwindow.repaint();
      cellautoapp.edittablewindow.edittablepanel.repaint();
      cellautoapp.colorbox.colorboxpanel.repaint();
    }
    if (ch == cellautokeys.save_to_stack_key) {
      boardstate.s = 1;
      cellautoapp.updatestatusbar();
    }
    if (ch == cellautokeys.edit_rows_mode_key) {
      cellautoapp.edit_rows_mode = !cellautoapp.edit_rows_mode;
      cellautoapp.updatestatusbar();
    }
    if (ch == cellautokeys.edit_notes_mode_key) {
      cellautoapp.edit_notes_mode = !cellautoapp.edit_notes_mode;
      cellautoapp.updatestatusbar();
    }
    if ((ch == '-')|(ch == '_')) {
        int x = cellautoapp.offsetx + (cellautoapp.cellautomainwindow.getWidth()/(cellautomainwindowc.sqsize*2));
        int y = cellautoapp.offsety + (cellautoapp.cellautomainwindow.getHeight()/(cellautomainwindowc.sqsize*2));


        cellautoapp.zoom(x,y,cellautoapp.ZOOM_OUT);
        //cellautoapp.cellautomainwindow.repaint();
    }
    if (ch == ' ') {
      cellautoapp.calc(1);
      
    }
    if (ch == cellautokeys.prev_color_key) {
        //select the prev color
        int selcolor = cellautoapp.colorbox.colorboxpanel.selcolor;
        int offset = cellautoapp.colorbox.colorboxpanel.offset;
        int numcolorsperrow = colorboxpanelc.numcolorsperrow;

        selcolor = (selcolor - 1 + cellautoapp.cellauto131.numcolors) % cellautoapp.cellauto131.numcolors;
        cellautoapp.colorlistindex = selcolor;
        int cd = selcolor - (offset*numcolorsperrow); 
        if (cd < 0){
            cellautoapp.colorbox.scrollbar.setValue(selcolor / numcolorsperrow);
        }      
        cellautoapp.colorbox.colorboxpanel.selcolor = selcolor;
        cellautoapp.colorbox.colorboxpanel.repaint();
        cellautoapp.updatestatusbar();
    }
    if (ch == cellautokeys.next_color_key){
        //select the next color
        int selcolor = cellautoapp.colorbox.colorboxpanel.selcolor;
        int offset = cellautoapp.colorbox.colorboxpanel.offset;
        int numcolorsperrow = colorboxpanelc.numcolorsperrow;
        
        selcolor = (selcolor + 1) % cellautoapp.cellauto131.numcolors;
        cellautoapp.colorlistindex = selcolor;
        cellautoapp.colorbox.colorboxpanel.repaint();
        int cd = selcolor - (offset*numcolorsperrow); 
        if (cd < 0){
            cellautoapp.colorbox.scrollbar.setValue(selcolor / numcolorsperrow);
        }
        cellautoapp.colorbox.colorboxpanel.selcolor = selcolor;
        cellautoapp.updatestatusbar();
    }
    
    if (a == 38){//move up
      cellautoapp.offsety = cellautoapp.offsety - cellautoapp.movestep;
    }
    if (a == 40){//move down
      cellautoapp.offsety = cellautoapp.offsety + cellautoapp.movestep;
    }
    if (a == 37){//move left
      cellautoapp.offsetx = cellautoapp.offsetx - cellautoapp.movestep;
    }
    if (a == 39){//move right
      cellautoapp.offsetx = cellautoapp.offsetx + cellautoapp.movestep;
    }
    cellautoapp.updatestatusbar();
    cellautoapp.cellautomainwindow.repaint();
  }
  public void keyReleased(KeyEvent e){
    //if (cellautoapp.verbose == true) {
    //  System.out.println("key release");
    //}
  }
  public void keyTyped(KeyEvent e){
    //if (cellautoapp.verbose == true) {
    //  System.out.println("key type");
    //}
  }

} 



class abstract_playlist_itemc {
  
  //abstract_playlist_item() {
  //}
  int get_num_lines() {return 0;}
  void read(int i,String line) {}
  String write(int i) {return null;}
  void loaditem(int tempo){}
  String getname(){return null;}
  String gettext(){return null;}
  static abstract_playlist_itemc make_new_item(String name) {
    if (name.equals("seed")) {
      return new seed_playlist_itemc();
    }
    return null;
  }
}

class seed_playlist_itemc extends abstract_playlist_itemc {
  long seed = 0;
  int advance_rows = 0;
  int tempo = 0;
  int music_key = 0;
  int width = 0;
  int height = 0;
  seed_playlist_itemc() {
    
  }
  seed_playlist_itemc(long seed,int advance_rows,int tempo,int music_key,int width,int height) {
    this.seed = seed;
    this.advance_rows = advance_rows;
    this.tempo = tempo;
    this.music_key = music_key;
    this.width = width;
    this.height = height;
  }
  void loaditem(int tempo2){
    int tempo3 = tempo;
    if (tempo2 > 0) {tempo3 = tempo2;}
    System.out.println("tempo used: " + tempo3);
    randomtabledialogc.nextbuttonclick(seed,seed,seed,advance_rows,tempo3,music_key,music_key,width,height,cellautoapp.PLAY_LIST_WINDOW);
  }
  int get_num_lines() {
    return 1;
  }
  void read(int i,String line) {
    String substr[] = line.split(",");
    seed = Long.valueOf(substr[0]).longValue();
    advance_rows = Integer.valueOf(substr[1]).intValue();
    tempo = Integer.valueOf(substr[2]).intValue();
    music_key = Integer.valueOf(substr[3]).intValue();
    width = Integer.valueOf(substr[4]).intValue();
    height = Integer.valueOf(substr[5]).intValue();
  }
  String write(int i) {
    String str = seed + "," + advance_rows + "," + tempo + "," + music_key + "," + width + "," + height;
    return str;
  }

  String getname() {
    return "seed";
  }  
  String gettext() {
    return "seed: " + seed;
  }
}
class playlistwindowc extends JFrame implements ActionListener, WindowListener {
  
  
  JList listbox;  
  
  JCheckBox tempo_checkbox;
  JTextField tempo_edit;
  
  JLabel seed_label;
  JTextField seed_edit;

  JLabel music_key_label;
  JTextField music_key_edit;
  
  JLabel advance_rows_label;
  JTextField advance_rows_edit;
  
  JButton new_seed_button;
  JButton add_seed_button;
  
  JButton play_button;
  JButton stop_button;

  JButton load_play_list_button;
  JButton save_play_list_button;
  
  JScrollPane listboxscroller;
  DefaultListModel list;  
  
  Vector itemlist;
  playlistwindowc(String Title){
    super(Title);    
    GridBagLayout gl = new GridBagLayout();
    this.getContentPane().setLayout(gl);
    GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1.0;
    c.weighty = 1.0;

    itemlist = new Vector();
    list = new DefaultListModel();
    listbox = new JList(list);
    listboxscroller = new JScrollPane(listbox);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(listboxscroller, c);
    this.getContentPane().add(listboxscroller);
    
    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1.0;
    c.weighty = 0.0;
    tempo_checkbox = new JCheckBox("use tempo:");
    c.gridwidth = 2;
    gl.setConstraints(tempo_checkbox, c);
    this.getContentPane().add(tempo_checkbox);

    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1.0;
    c.weighty = 0.0;
    tempo_edit = new JTextField("24");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(tempo_edit, c);
    this.getContentPane().add(tempo_edit);
    

    advance_rows_label = new JLabel();
    advance_rows_label.setText("row to start at:");
    c.gridwidth = 2;
    gl.setConstraints(advance_rows_label, c);
    this.getContentPane().add(advance_rows_label);

    advance_rows_edit = new JTextField();
    advance_rows_edit.setText("0");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(advance_rows_edit, c);
    this.getContentPane().add(advance_rows_edit);

    
    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1.0;
    c.weighty = 0.0;
    seed_label = new JLabel("seed: ");
    c.gridwidth = 2;
    gl.setConstraints(seed_label, c);
    this.getContentPane().add(seed_label);

    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1.0;
    c.weighty = 0.0;
    seed_edit = new JTextField("0");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(seed_edit, c);
    this.getContentPane().add(seed_edit);
    

    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1.0;
    c.weighty = 0.0;
    music_key_label = new JLabel("music key: ");
    c.gridwidth = 2;
    gl.setConstraints(music_key_label, c);
    this.getContentPane().add(music_key_label);

    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1.0;
    c.weighty = 0.0;
    music_key_edit = new JTextField("0");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(music_key_edit, c);
    this.getContentPane().add(music_key_edit);

    
    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1.0;
    c.weighty = 0.0;
    play_button = new JButton("play");
    play_button.setActionCommand("play");
    play_button.addActionListener(this);
    c.gridwidth = 2;
    gl.setConstraints(play_button, c);
    this.getContentPane().add(play_button);

    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1.0;
    c.weighty = 0.0;
    stop_button = new JButton("stop");
    stop_button.setActionCommand("stop");
    stop_button.addActionListener(this);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(stop_button, c);
    this.getContentPane().add(stop_button);

    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1.0;
    c.weighty = 0.0;
    new_seed_button = new JButton("new seed");
    new_seed_button.setActionCommand("new seed");
    new_seed_button.addActionListener(this);
    c.gridwidth = 2;
    gl.setConstraints(new_seed_button, c);
    this.getContentPane().add(new_seed_button);

    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1.0;
    c.weighty = 0.0;
    add_seed_button = new JButton("add seed");
    add_seed_button.setActionCommand("add seed");
    add_seed_button.addActionListener(this);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(add_seed_button, c);
    this.getContentPane().add(add_seed_button);

    
    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1.0;
    c.weighty = 0.0;
    load_play_list_button = new JButton("load");
    load_play_list_button.setActionCommand("load");
    load_play_list_button.addActionListener(this);
    c.gridwidth = 2;
    gl.setConstraints(load_play_list_button, c);
    this.getContentPane().add(load_play_list_button);

    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1.0;
    c.weighty = 0.0;
    save_play_list_button = new JButton("save");
    save_play_list_button.setActionCommand("save");
    save_play_list_button.addActionListener(this);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(save_play_list_button, c);
    this.getContentPane().add(save_play_list_button);

    
    addWindowListener(this);

    this.pack();    
    
  }
  void loadplaylist(String filename) {
    try{
      BufferedReader r = new BufferedReader(new FileReader(filename));
      String line = null;
      line = r.readLine();
      line = r.readLine();
      double version = Double.valueOf(line).doubleValue();
      line = r.readLine();
      list.clear();
      itemlist.clear();
      while (line != null){
        line.trim();
        abstract_playlist_itemc i = abstract_playlist_itemc.make_new_item(line);
        int n = i.get_num_lines();
        for (int i2 = 0;i2 < n;i2++) {
          line = r.readLine();
          if (line == null) {break;}
          i.read(i2,line);
        }
        itemlist.add(i);
        list.addElement(i.gettext());        
        line = r.readLine();
      }
      r.close();
    } catch (IOException e){
      e.printStackTrace();    
    }
  }
  void saveplaylist(String filename) {
    try{
      PrintWriter p = new PrintWriter(new BufferedWriter(new FileWriter(filename,true)));
      p.println(cellautoapp.programname + " play list");
      p.println(cellautoapp.programversion3);
      for (int i = 0;i < itemlist.size();i++){
        abstract_playlist_itemc i2 = (abstract_playlist_itemc) itemlist.get(i);
        p.println(i2.getname());
        int n = i2.get_num_lines();
        for (int i3 = 0;i3 < n;i3++) {
          p.println(i2.write(i3));
        }
      }
      p.close();
    } catch (Exception e){
      e.printStackTrace();
    }
    
  }
  void playsong() {
      int i = listbox.getSelectedIndex();      
      int tempo = cellautoapp.tempo;
      if (i > -1) {
        abstract_playlist_itemc i2 = (abstract_playlist_itemc) itemlist.get(i);
        if (tempo_checkbox.isSelected()) {
          tempo = Integer.parseInt(tempo_edit.getText().trim());
        }
        cellautoapp.playlistrunning = false;
        i2.loaditem(tempo);
        cellautoapp.playlistrunning = true;
      }  
  }
  void showoptions() {
    tempo_edit.setText(Integer.toString(cellautoapp.tempo));
    seed_edit.setText(Long.toString(cellautoapp.rule_table_seed));
  }
  public void windowActivated(WindowEvent e){}
  public void windowClosed(WindowEvent e){}
  public void windowClosing(WindowEvent e){
    cellautoapp.playlistrunning = false;
    cellautoapp.note_player.stop_Sequencer();
  }
  public void windowDeactivated(WindowEvent e){}  
  public void windowDeiconified(WindowEvent e){}
  public void windowIconified(WindowEvent e){}
  public void windowOpened(WindowEvent e){}    
  public void actionPerformed(ActionEvent e) {
    String action = e.getActionCommand();    
    System.out.println(action);
    if (action.equals("load")) {
      FileDialog f = new FileDialog(this,"game open",FileDialog.LOAD);
      f.show();
      if (f.getFile() != null) {
        String filename = f.getDirectory() + f.getFile();        
        loadplaylist(filename);
      }
    }
    if (action.equals("save")) {
      FileDialog f = new FileDialog(this,"game save",FileDialog.SAVE);
      f.show();
      if (f.getFile() != null) {
        String filename = f.getDirectory() + f.getFile();        
        String filetype = cellautoapp.getfiletype(filename);
        if (filetype.equals("lst") == false){
          filename = filename + ".lst";
        }
        File f2 = new File(filename);
        if (f2.exists() == false){
          saveplaylist(filename);
        } else {
          JOptionPane.showMessageDialog(cellautoapp.cellautowindowframe,new JLabel("file already exists"),"error",JOptionPane.ERROR_MESSAGE);          
        }        
      }
    }
    if (action.equals("play")) {
      playsong();
    }         
    if (action.equals("stop")) {
      cellautoapp.playlistrunning = false;
      cellautoapp.note_player.stop_Sequencer();
    }
    if (action.equals("new seed")) {
      boardc b = cellautoapp.board;
      int tempo = cellautoapp.tempo;
      int music_key = Integer.parseInt(music_key_edit.getText().trim());
      int music_key2 = cellautoapp.key_to_play_in;
      int advance_rows = Integer.parseInt(advance_rows_edit.getText().trim());
      if (tempo_checkbox.isSelected()) {
        tempo = Integer.parseInt(tempo_edit.getText().trim());
      }
      cellautoapp.rule_table_seed = Long.parseLong(seed_edit.getText().trim());      
      cellautoapp.playlistrunning = false;
      cellautoapp.update_rule_table_seed();  
      randomtabledialogc.nextbuttonclick(cellautoapp.rule_table_seed,cellautoapp.rule_table_seed,cellautoapp.rule_table_seed,advance_rows,tempo,music_key + music_key2,music_key + music_key2,b.width,b.height,cellautoapp.PLAY_LIST_WINDOW);
      if (cellautoapp.optionsdialog != null) {
        cellautoapp.optionsdialog.seededit2.setText(Long.toString(cellautoapp.rule_table_seed));
      }
      seed_edit.setText(Long.toString(cellautoapp.rule_table_seed));
    }
    if (action.equals("add seed")) {
      boardc b = cellautoapp.board;
      int tempo = cellautoapp.tempo;
      int music_key = Integer.parseInt(music_key_edit.getText().trim());
      int advance_rows = 0;
      seed_playlist_itemc i = new seed_playlist_itemc(cellautoapp.rule_table_seed,advance_rows,tempo,cellautoapp.key_to_play_in,b.width,b.height);
      itemlist.add(i);
      list.addElement(i.gettext());
    }

  }  
}

class randomtabledialogc extends JFrame implements ActionListener, WindowListener {
  JCheckBox seedcheckbox1;
  JTextField seededit1;
  JCheckBox seedcheckbox2;
  JTextField seededit2;
  JCheckBox seedcheckbox3;
  JTextField seededit3;

  JLabel fromkeylabel;
  JTextField fromkeyedit;
  JLabel tokeylabel;
  JTextField tokeyedit;
  JLabel advance_rows_label;
  JTextField advance_rows_edit;

  JLabel play_at_tempolabel;
  JTextField play_at_tempoedit;
  JLabel output_at_tempolabel;
  JTextField output_at_tempoedit;
  JLabel file_prefix_label;
  JTextField file_prefix_edit;
  JLabel file_num_label;
  JTextField file_num_edit;
  
  JButton nextbutton;
  JButton outputbutton;
  
  randomtabledialogc(String Title){
    super(Title);
    GridBagLayout gl = new GridBagLayout();
    this.getContentPane().setLayout(gl);
    GridBagConstraints c = new GridBagConstraints();

    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1.0;
    c.weighty = 1.0;

    
    seedcheckbox1 = new JCheckBox();
    seedcheckbox1.setText("increment board song seed:");
    c.gridwidth = 2;
    gl.setConstraints(seedcheckbox1, c);
    this.getContentPane().add(seedcheckbox1);
    seedcheckbox1.setSelected(true);
    
    seededit1 = new JTextField();
    seededit1.setText("1");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(seededit1, c);
    this.getContentPane().add(seededit1);
    
    
    seedcheckbox2 = new JCheckBox();
    seedcheckbox2.setText("increment rule table seed:");
    c.gridwidth = 2;
    gl.setConstraints(seedcheckbox2, c);
    this.getContentPane().add(seedcheckbox2);
    seedcheckbox2.setSelected(true);
    
    seededit2 = new JTextField();
    seededit2.setText("1");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(seededit2, c);
    this.getContentPane().add(seededit2);
    
    
    seedcheckbox3 = new JCheckBox();
    seedcheckbox3.setText("increment colored note seed:");
    c.gridwidth = 2;
    gl.setConstraints(seedcheckbox3, c);
    this.getContentPane().add(seedcheckbox3);
    seedcheckbox3.setSelected(true);
    
    seededit3 = new JTextField();
    seededit3.setText("1");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(seededit3, c);
    this.getContentPane().add(seededit3);
    
    
    fromkeylabel = new JLabel();
    fromkeylabel.setText("from key:");
    c.gridwidth = 2;
    gl.setConstraints(fromkeylabel, c);
    this.getContentPane().add(fromkeylabel);
  
    fromkeyedit = new JTextField();
    fromkeyedit.setText("1");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(fromkeyedit, c);
    this.getContentPane().add(fromkeyedit);

    
    tokeylabel = new JLabel();
    tokeylabel.setText("to key:");
    c.gridwidth = 2;
    gl.setConstraints(tokeylabel, c);
    this.getContentPane().add(tokeylabel);

    tokeyedit = new JTextField();
    tokeyedit.setText("1");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(tokeyedit, c);
    this.getContentPane().add(tokeyedit);

    
    advance_rows_label = new JLabel();
    advance_rows_label.setText("row to start at:");
    c.gridwidth = 2;
    gl.setConstraints(advance_rows_label, c);
    this.getContentPane().add(advance_rows_label);

    advance_rows_edit = new JTextField();
    advance_rows_edit.setText("0");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(advance_rows_edit, c);
    this.getContentPane().add(advance_rows_edit);

    
    play_at_tempolabel = new JLabel();
    play_at_tempolabel.setText("play at tempo:");
    c.gridwidth = 2;
    gl.setConstraints(play_at_tempolabel, c);
    this.getContentPane().add(play_at_tempolabel);

    play_at_tempoedit = new JTextField();
    play_at_tempoedit.setText("1");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(play_at_tempoedit, c);
    this.getContentPane().add(play_at_tempoedit);


    output_at_tempolabel = new JLabel();
    output_at_tempolabel.setText("output at tempo:");
    c.gridwidth = 2;
    gl.setConstraints(output_at_tempolabel, c);
    this.getContentPane().add(output_at_tempolabel);
    
    output_at_tempoedit = new JTextField();
    output_at_tempoedit.setText("1");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(output_at_tempoedit, c);
    this.getContentPane().add(output_at_tempoedit);

    
    file_prefix_label = new JLabel();
    file_prefix_label.setText("file prefix:");
    c.gridwidth = 2;
    gl.setConstraints(file_prefix_label, c);
    this.getContentPane().add(file_prefix_label);
  
    int i = cellautoapp.rand2.nextInt(1000000);
    file_prefix_edit = new JTextField();
    file_prefix_edit.setText("music" + i);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(file_prefix_edit, c);
    this.getContentPane().add(file_prefix_edit);

    
    file_num_label = new JLabel();
    file_num_label.setText("file num:");
    c.gridwidth = 2;
    gl.setConstraints(file_num_label, c);
    this.getContentPane().add(file_num_label);
  
    file_num_edit = new JTextField();
    file_num_edit.setText("1");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(file_num_edit, c);
    this.getContentPane().add(file_num_edit);

    nextbutton = new JButton();
    nextbutton.setText("next");
    nextbutton.setActionCommand("nextbuttonclick");
    nextbutton.addActionListener(this);
    c.gridwidth = 2;
    gl.setConstraints(nextbutton, c);
    this.getContentPane().add(nextbutton);

    outputbutton = new JButton();
    outputbutton.setText("output");
    outputbutton.setActionCommand("outputbuttonclick");
    outputbutton.addActionListener(this);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(outputbutton, c);
    this.getContentPane().add(outputbutton);
    
    addWindowListener(this);
    this.pack();
  }
  void showoptions() {
    seededit1.setText(Long.toString(cellautoapp.board_song_seed));
    seededit2.setText(Long.toString(cellautoapp.rule_table_seed));
    seededit3.setText(Long.toString(cellautoapp.colored_note_seed));
    fromkeyedit.setText(Integer.toString(cellautoapp.key_to_play_in));
    tokeyedit.setText(Integer.toString(cellautoapp.key_to_play_in));
    play_at_tempoedit.setText(Integer.toString(cellautoapp.tempo));
    output_at_tempoedit.setText(Integer.toString(cellautoapp.tempo));
  }
  static int get_note_distance(int n1,int n2) {
    int d = n2-n1;
    if (d < 0) {d = -d;}
    return d;
  }
  static void change_note(int x,int y,int n) {
    int sq = cellautoapp.board.getsq(x,y);
    cellautorulec c = cellautoapp.cellauto131;
    if (sq > 0) {
      int i = c.getsum(cellautoapp.board,x,y);
      int i2 = cellautoapp.rand.nextInt(c.numcolors-1)+1;
      c.setlookuptablewithindex(i,i2);
      cellautoapp.board.setsq(x,y,i2);
    }
  }
  static void nextbuttonclick(long seed1,long seed2,long seed3,int advance_rows,int tempo,int from_key,int to_key,int width,int height,int f) {
    cellautoapp.playlistrunning = false;
    cellautoapp.rand2.setSeed(seed2);
    cellautoapp.randomtable();
    if ((f & 1) == cellautoapp.RANDOM_TABLE_WINDOW) {cellautoapp.rand2.setSeed(seed3);}
    cellautoapp.randomnotes();
    boardc b = cellautoapp.board;
    if ((f & 1) == cellautoapp.RANDOM_TABLE_WINDOW) {cellautoapp.rand2.setSeed(seed1);}
    cellautoapp.board = boardc.new_random_board(cellautoapp.rand2,width,height,0,width);
    cellautoapp.key_to_play_in = from_key+cellautoapp.rand2.nextInt(to_key-from_key+1);
    if (cellautoapp.optionsdialog != null) {
      cellautoapp.optionsdialog.key_to_play_in_edit.setText(Integer.toString(cellautoapp.key_to_play_in));      
    }
    for (int i = 0;i < advance_rows;i++) {
      int lastnote = 0;
      cellautoapp.calc(1);
      //for (int j = 0;j < cellautoapp.board.width;j++){
      //  int note = cellautoapp.board.getsq(i+1,j);
      //  if (note != 0) {
      //    if (get_note_distance(note,lastnote) > cellautoapp.max_note_distance) {
      //      change_note(i+1,j,lastnote);
      //    }
      //    lastnote = note;
      //  }
      //}
    }
    cellautoapp.calc(cellautoapp.board.height);
    cellautoapp.CA_Player.clearnotelist();
    for (int i = 0;i < cellautoapp.board.height;i++){
      int i2 = (cellautoapp.board.generation+i+1) % cellautoapp.board.height;
      cellautoapp.CA_Player.play_row(cellautoapp.board,i2);
    }
    cellautoapp.CA_Player.play_last_note();
    notelistc l = cellautoapp.CA_Player.getnotelist();
    l.addchordstonotes();
    cellautoapp.note_player.playnotelist(l,tempo);
    cellautoapp.updatestatusbar();
    cellautoapp.cellautomainwindow.repaint();
    cellautoapp.edittablewindow.edittablepanel.repaint();
    cellautoapp.colorbox.colorboxpanel.repaint();
  }

  void outputbuttonclick() {
    String file_prefix = file_prefix_edit.getText();
    int file_num = Integer.parseInt(file_num_edit.getText().trim());
    int tempo = Integer.parseInt(output_at_tempoedit.getText().trim());

    cellautoapp.CA_Player.clearnotelist();
    for (int i = 0;i < cellautoapp.board.height;i++){
      int i2 = (cellautoapp.board.generation+i+1) % cellautoapp.board.height;
      cellautoapp.CA_Player.play_row(cellautoapp.board,i2);
    }
    cellautoapp.CA_Player.play_last_note();
    notelistc l = cellautoapp.CA_Player.getnotelist();
    l.addchordstonotes();
    File f = new File("CAMusic/" + file_prefix + file_num + ".midi");
    if (f.exists() == false){
      cellautoapp.note_player.output_notelist_to_file(l,f,tempo);                             
    } else {
      JOptionPane.showMessageDialog(cellautoapp.cellautowindowframe,new JLabel("file already exists"),"error",JOptionPane.ERROR_MESSAGE);          
    }      
    f = new File("CAMusic/" + file_prefix + file_num + ".mca");
    if (f.exists() == false){
      try {
        cellautoapp.savetext(new BufferedWriter(new FileWriter(f,true)));
      } catch (Exception err){
        err.printStackTrace();        
      }
    } else {
      JOptionPane.showMessageDialog(cellautoapp.cellautowindowframe,new JLabel("file already exists"),"error",JOptionPane.ERROR_MESSAGE);          
    }      
    file_num = file_num + 1;
    file_num_edit.setText(Integer.toString(file_num));
  }
  
  public void windowActivated(WindowEvent e){}
  public void windowClosed(WindowEvent e){}
  public void windowClosing(WindowEvent e){
    cellautoapp.note_player.stop_Sequencer();
  }
  public void windowDeactivated(WindowEvent e){}  
  public void windowDeiconified(WindowEvent e){}
  public void windowIconified(WindowEvent e){}
  public void windowOpened(WindowEvent e){}  
  public void actionPerformed(ActionEvent e) {
    String action = e.getActionCommand();    
    //System.out.println(action);
    if (action.equals("nextbuttonclick")) {
      boardc b = cellautoapp.board;
      int advance_rows = Integer.parseInt(advance_rows_edit.getText().trim());
      int tempo = Integer.parseInt(play_at_tempoedit.getText().trim());
      int from_key = Integer.parseInt(fromkeyedit.getText().trim());
      int to_key = Integer.parseInt(tokeyedit.getText().trim());
      cellautoapp.board_song_seed = Long.parseLong(seededit1.getText().trim());
      cellautoapp.rule_table_seed = Long.parseLong(seededit2.getText().trim());
      cellautoapp.colored_note_seed = Long.parseLong(seededit3.getText().trim());
      if (seedcheckbox1.isSelected()) {
        cellautoapp.update_board_song_seed();
      }
      if (seedcheckbox2.isSelected()) {
        cellautoapp.update_rule_table_seed();  
      }
      if (seedcheckbox3.isSelected()) {
        cellautoapp.update_colored_note_seed();
      }
      nextbuttonclick(cellautoapp.board_song_seed,cellautoapp.rule_table_seed,cellautoapp.colored_note_seed,advance_rows,tempo,from_key,to_key,b.width,b.height,cellautoapp.RANDOM_TABLE_WINDOW);
      if (cellautoapp.optionsdialog != null) {
        cellautoapp.optionsdialog.seededit1.setText(Long.toString(cellautoapp.board_song_seed));
        cellautoapp.optionsdialog.seededit2.setText(Long.toString(cellautoapp.rule_table_seed));
        cellautoapp.optionsdialog.seededit3.setText(Long.toString(cellautoapp.colored_note_seed));
      }
      seededit1.setText(Long.toString(cellautoapp.board_song_seed));
      seededit2.setText(Long.toString(cellautoapp.rule_table_seed));
      seededit3.setText(Long.toString(cellautoapp.colored_note_seed));
    }
    if (action.equals("outputbuttonclick")) {
      outputbuttonclick();
    }
    
  } 
}
class randomize_table_dialogc extends JFrame implements ActionListener {
  JCheckBox seedcheckbox;
  JTextField seededit;
  JButton randombutton;
  JButton closebutton;
  randomize_table_dialogc(String Title) {
    super(Title);
    GridBagLayout gl = new GridBagLayout();
    this.getContentPane().setLayout(gl);
    GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1.0;
    c.weighty = 1.0;
    //this.setBounds(0,0,600,300);
    seedcheckbox = new JCheckBox("increment rule table seed:");
    seedcheckbox.setSelected(true);
    addcomponent(gl,seedcheckbox,2,c);

    seededit = new JTextField("0");
    addcomponent(gl,seededit,GridBagConstraints.REMAINDER,c);

    randombutton = new JButton("randomize table");
    randombutton.setActionCommand("randombuttonclick");
    randombutton.addActionListener(this);
    addcomponent(gl,randombutton,2,c);

    closebutton = new JButton("close");
    closebutton.setActionCommand("closebuttonclick");
    closebutton.addActionListener(this);
    addcomponent(gl,closebutton,GridBagConstraints.REMAINDER,c);

    this.pack();
    
  }
  void addcomponent(GridBagLayout gl,JComponent c,int c2,GridBagConstraints c3) {

    c3.gridwidth = c2;
    gl.setConstraints(c, c3);
    this.getContentPane().add(c);
  }
  public void actionPerformed(ActionEvent e) {
    String action = e.getActionCommand();
    if (action.equals("randombuttonclick")) {
      if (seedcheckbox.isSelected()) {
        cellautoapp.update_rule_table_seed();
      }
      cellautoapp.rand2.setSeed(cellautoapp.rule_table_seed);
      seededit.setText(Long.toString(cellautoapp.rule_table_seed));
      cellautoapp.randomtable();
      cellautoapp.edittablewindow.repaint();      
    }
    if (action.equals("closebuttonclick")) {
      this.setVisible(false);
    }
  }

}
class randomize_notes_dialogc extends JFrame implements ActionListener {
  JCheckBox seedcheckbox;
  JTextField seededit;
  JButton randombutton;
  JButton closebutton;
  
  randomize_notes_dialogc(String Title) {
    super(Title);
    GridBagLayout gl = new GridBagLayout();
    this.getContentPane().setLayout(gl);
    GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1.0;
    c.weighty = 1.0;
    //this.setBounds(0,0,300,200);
    seedcheckbox = new JCheckBox("increment colored note seed:");
    seedcheckbox.setSelected(true);
    addcomponent(gl,seedcheckbox,2,c);

    seededit = new JTextField("0");
    addcomponent(gl,seededit,GridBagConstraints.REMAINDER,c);

    randombutton = new JButton("randomize notes");
    randombutton.setActionCommand("randombuttonclick");
    randombutton.addActionListener(this);
    addcomponent(gl,randombutton,2,c);

    closebutton = new JButton("close");
    closebutton.setActionCommand("closebuttonclick");
    closebutton.addActionListener(this);
    addcomponent(gl,closebutton,GridBagConstraints.REMAINDER,c);

    this.pack();
    
  }
  void addcomponent(GridBagLayout gl,JComponent c,int c2,GridBagConstraints c3) {

    c3.gridwidth = c2;
    gl.setConstraints(c, c3);
    this.getContentPane().add(c);
  }
  public void actionPerformed(ActionEvent e) {
    String action = e.getActionCommand();
    if (action.equals("randombuttonclick")) {
      if (seedcheckbox.isSelected()) {
        cellautoapp.update_colored_note_seed();
      }
      cellautoapp.rand2.setSeed(cellautoapp.colored_note_seed);
      cellautoapp.randomnotes();
      cellautoapp.board.updateboard2(cellautoapp.cellauto131);
      seededit.setText(Long.toString(cellautoapp.colored_note_seed));
      cellautoapp.edittablewindow.repaint();
      cellautoapp.cellautomainwindow.repaint();
      
    }
    if (action.equals("closebuttonclick")) {
      this.setVisible(false);
      
    }
  }
}


class optionsdialogc extends JFrame implements ActionListener {
  JLabel tempolabel;
  JTextField tempoedit;
  JLabel num_rows_to_play_label;
  JTextField num_rows_to_play_edit;
  JLabel seedlabel1;
  JTextField seededit1;
  JLabel seedlabel2;
  JTextField seededit2;
  JLabel seedlabel3;
  JTextField seededit3;
  JLabel min_note_length_label;
  JTextField min_note_length_edit;
  JLabel empty_cell_space_bonus_label;
  JTextField empty_cell_space_bonus_edit;
  JLabel num_notes_to_play_label;
  JTextField num_notes_to_play_edit;
  JLabel key_to_play_in_label;
  JTextField key_to_play_in_edit;
  JLabel melody_instrument_label;
  JTextField melody_instrument_edit;
  JLabel chord_instrument_label;
  JTextField chord_instrument_edit;
  JLabel chord_volume_label;
  JTextField chord_volume_edit;
  JLabel octavecombobox_label;
  JComboBox octavecombobox;
  JLabel chord_octave_combobox_label;
  JComboBox chord_octave_combobox;
  JCheckBox auto_update_rule_table_checkbox;
  
  JButton okbutton;
  JButton cancelbutton;
  String[] Octave_Strings = {"lower octave","upper octave","both octaves"};
  String[] Chord_Octave_Strings = {"center octave","best center octave","best octave"};
  optionsdialogc(String Title) {
    super(Title);
    GridBagLayout gl = new GridBagLayout();
    this.getContentPane().setLayout(gl);
    GridBagConstraints c = new GridBagConstraints();

    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1.0;
    c.weighty = 1.0;

    tempolabel = new JLabel();
    tempolabel.setText("tempo");
    c.gridwidth = 2;
    gl.setConstraints(tempolabel, c);
    this.getContentPane().add(tempolabel);
  
    tempoedit = new JTextField();
    tempoedit.setText("1");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(tempoedit, c);
    this.getContentPane().add(tempoedit);

    num_rows_to_play_label = new JLabel();
    num_rows_to_play_label.setText("num rows to play");
    c.gridwidth = 2;
    gl.setConstraints(num_rows_to_play_label, c);
    this.getContentPane().add(num_rows_to_play_label);
  
    num_rows_to_play_edit = new JTextField();
    num_rows_to_play_edit.setText("1");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(num_rows_to_play_edit, c);
    this.getContentPane().add(num_rows_to_play_edit);

    seedlabel1 = new JLabel();
    seedlabel1.setText("board song seed:");
    c.gridwidth = 2;
    gl.setConstraints(seedlabel1, c);
    this.getContentPane().add(seedlabel1);
  
    seededit1 = new JTextField();
    seededit1.setText("0");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(seededit1, c);
    this.getContentPane().add(seededit1);

    
    seedlabel2 = new JLabel();
    seedlabel2.setText("rule table seed:");
    c.gridwidth = 2;
    gl.setConstraints(seedlabel2, c);
    this.getContentPane().add(seedlabel2);
  
    seededit2 = new JTextField();
    seededit2.setText("0");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(seededit2, c);
    this.getContentPane().add(seededit2);


    seedlabel3 = new JLabel();
    seedlabel3.setText("colored note seed:");
    c.gridwidth = 2;
    gl.setConstraints(seedlabel3, c);
    this.getContentPane().add(seedlabel3);
  
    seededit3 = new JTextField();
    seededit3.setText("0");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(seededit3, c);
    this.getContentPane().add(seededit3);

    
    min_note_length_label = new JLabel();
    min_note_length_label.setText("min note length");
    c.gridwidth = 2;
    gl.setConstraints(min_note_length_label, c);
    this.getContentPane().add(min_note_length_label);
  
    min_note_length_edit = new JTextField();
    min_note_length_edit.setText("0");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(min_note_length_edit, c);
    this.getContentPane().add(min_note_length_edit);

    
    empty_cell_space_bonus_label = new JLabel();
    empty_cell_space_bonus_label.setText("empty space bonus length");
    c.gridwidth = 2;
    gl.setConstraints(empty_cell_space_bonus_label, c);
    this.getContentPane().add(empty_cell_space_bonus_label);
  
    empty_cell_space_bonus_edit = new JTextField();
    empty_cell_space_bonus_edit.setText("0");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(empty_cell_space_bonus_edit, c);
    this.getContentPane().add(empty_cell_space_bonus_edit);

    
    num_notes_to_play_label = new JLabel();
    num_notes_to_play_label.setText("number of notes to play at a time");
    c.gridwidth = 2;
    gl.setConstraints(num_notes_to_play_label, c);
    this.getContentPane().add(num_notes_to_play_label);
  
    num_notes_to_play_edit = new JTextField();
    num_notes_to_play_edit.setText("1");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(num_notes_to_play_edit, c);
    this.getContentPane().add(num_notes_to_play_edit);
    


    octavecombobox_label = new JLabel();
    octavecombobox_label.setText("play");
    c.gridwidth = 2;
    gl.setConstraints(octavecombobox_label, c);
    this.getContentPane().add(octavecombobox_label);
  
    octavecombobox = new JComboBox(Octave_Strings);
    //octavecombobox.setText("0");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(octavecombobox, c);
    this.getContentPane().add(octavecombobox);


    chord_octave_combobox_label = new JLabel();
    chord_octave_combobox_label.setText("select chord octave:");
    c.gridwidth = 2;
    gl.setConstraints(chord_octave_combobox_label, c);
    this.getContentPane().add(chord_octave_combobox_label);

    chord_octave_combobox = new JComboBox(Chord_Octave_Strings);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(chord_octave_combobox, c);
    this.getContentPane().add(chord_octave_combobox);
    
    
    key_to_play_in_label = new JLabel();
    key_to_play_in_label.setText("key to play in");
    c.gridwidth = 2;
    gl.setConstraints(key_to_play_in_label, c);
    this.getContentPane().add(key_to_play_in_label);

    key_to_play_in_edit = new JTextField();
    key_to_play_in_edit.setText("0");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(key_to_play_in_edit, c);
    this.getContentPane().add(key_to_play_in_edit);
  

    melody_instrument_label = new JLabel();
    melody_instrument_label.setText("melody instrument");
    c.gridwidth = 2;
    gl.setConstraints(melody_instrument_label, c);
    this.getContentPane().add(melody_instrument_label);

    melody_instrument_edit = new JTextField();
    melody_instrument_edit.setText("1");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(melody_instrument_edit, c);
    this.getContentPane().add(melody_instrument_edit);

    

    chord_instrument_label = new JLabel();
    chord_instrument_label.setText("chord instrument");
    c.gridwidth = 2;
    gl.setConstraints(chord_instrument_label, c);
    this.getContentPane().add(chord_instrument_label);

    chord_instrument_edit = new JTextField();
    chord_instrument_edit.setText("1");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(chord_instrument_edit, c);
    this.getContentPane().add(chord_instrument_edit);
  

    chord_volume_label = new JLabel();
    chord_volume_label.setText("chord volume");
    c.gridwidth = 2;
    gl.setConstraints(chord_volume_label, c);
    this.getContentPane().add(chord_volume_label);

    chord_volume_edit = new JTextField();
    chord_volume_edit.setText("0");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(chord_volume_edit, c);
    this.getContentPane().add(chord_volume_edit);
    
    auto_update_rule_table_checkbox = new JCheckBox("auto update rule table");
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(auto_update_rule_table_checkbox, c);
    this.getContentPane().add(auto_update_rule_table_checkbox);
    
    
    okbutton = new JButton();
    okbutton.setText("ok");
    okbutton.setActionCommand("okbuttonclick");
    okbutton.addActionListener(this);
    c.gridwidth = 2;
    gl.setConstraints(okbutton, c);
    this.getContentPane().add(okbutton);

    cancelbutton = new JButton();
    cancelbutton.setText("cancel");
    cancelbutton.setActionCommand("cancelbuttonclick");
    cancelbutton.addActionListener(this);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gl.setConstraints(cancelbutton, c);
    this.getContentPane().add(cancelbutton);

    this.pack();

  }
  //this function displays the options
  void getoptions(){
    tempoedit.setText(Integer.toString(cellautoapp.tempo));
    seededit1.setText(Long.toString(cellautoapp.board_song_seed));
    seededit2.setText(Long.toString(cellautoapp.rule_table_seed));
    seededit3.setText(Long.toString(cellautoapp.colored_note_seed));
    num_rows_to_play_edit.setText(Integer.toString(cellautoapp.num_rows_to_play));
    min_note_length_edit.setText(Integer.toString(cellautoapp.min_note_length));
    empty_cell_space_bonus_edit.setText(Integer.toString(cellautoapp.empty_cell_space_bonus));
    num_notes_to_play_edit.setText(Integer.toString(cellautoapp.num_notes_at_a_time));
    octavecombobox.setSelectedIndex(cellautoapp.play_octaves);
    chord_octave_combobox.setSelectedIndex(cachord.select_chord_octave);
    key_to_play_in_edit.setText(Integer.toString(cellautoapp.key_to_play_in));
    melody_instrument_edit.setText(Integer.toString(cellautoapp.melody_instrument));
    chord_instrument_edit.setText(Integer.toString(cellautoapp.chord_instrument));
    chord_volume_edit.setText(Integer.toString(cellautoapp.chord_volume));
    auto_update_rule_table_checkbox.setSelected(cellautoapp.auto_update_rule_table);
  }
  
  //this function updates the options
  void updateoptions(){
    cellautoapp.tempo = Integer.valueOf(tempoedit.getText().trim()).intValue();
    cellautoapp.board_song_seed = Long.valueOf(seededit1.getText().trim()).longValue();
    cellautoapp.rule_table_seed = Long.valueOf(seededit2.getText().trim()).longValue();
    cellautoapp.colored_note_seed = Long.valueOf(seededit3.getText().trim()).longValue();
    cellautoapp.num_rows_to_play = Integer.valueOf(num_rows_to_play_edit.getText().trim()).intValue();
    cellautoapp.min_note_length = Integer.valueOf(min_note_length_edit.getText().trim()).intValue();
    cellautoapp.empty_cell_space_bonus = Integer.valueOf(empty_cell_space_bonus_edit.getText().trim()).intValue();
    cellautoapp.key_to_play_in = Integer.valueOf(key_to_play_in_edit.getText().trim()).intValue();
    if (cellautoapp.key_to_play_in > 240) {cellautoapp.key_to_play_in = 240;}
    if (cellautoapp.key_to_play_in < -240) {cellautoapp.key_to_play_in = -240;}
    cellautoapp.melody_instrument = Integer.valueOf(melody_instrument_edit.getText().trim()).intValue();
    cellautoapp.chord_instrument = Integer.valueOf(chord_instrument_edit.getText().trim()).intValue();
    cellautoapp.chord_volume = Integer.valueOf(chord_volume_edit.getText().trim()).intValue();
    cellautoapp.num_notes_at_a_time = Integer.valueOf(num_notes_to_play_edit.getText().trim()).intValue();
    cellautoapp.play_octaves = octavecombobox.getSelectedIndex();
    cellautoapp.auto_update_rule_table = auto_update_rule_table_checkbox.isSelected();
    cachord.select_chord_octave = chord_octave_combobox.getSelectedIndex();
    cellautoapp.cellautomainwindow.repaint();
    cellautoapp.edittablewindow.edittablepanel.repaint();
    cellautoapp.colorbox.colorboxpanel.repaint();  
    cellautoapp.updatestatusbar();
  }
  public void actionPerformed(ActionEvent e) {
    String action = e.getActionCommand();
    System.out.println(action);
    if (action.equals("okbuttonclick")) {
      updateoptions();
      this.setVisible(false);
    }
    if (action.equals("cancelbuttonclick")) {
      this.setVisible(false);
    }
  }
}

class colorboxc extends JFrame{
  colorboxpanelc colorboxpanel;
  JScrollBar scrollbar;
  colorboxadjlisc adjlis;
  static boolean colorboxautoscroll = true;
  colorboxc(String Title){
    super(Title);
    colorboxpanel = new colorboxpanelc();
    scrollbar = new JScrollBar(JScrollBar.VERTICAL);
    scrollbar.setMinimum(0);
    scrollbar.setMaximum(64);
    scrollbar.setVisibleAmount(0);
    adjlis = new colorboxadjlisc();
    scrollbar.addAdjustmentListener(adjlis);
    SpringLayout spl = new SpringLayout();         
    this.getContentPane().setLayout(spl);
    spl.putConstraint(SpringLayout.WEST,scrollbar,0,SpringLayout.EAST,colorboxpanel);
    spl.putConstraint(SpringLayout.SOUTH,colorboxpanel,0,SpringLayout.SOUTH,this.getContentPane());
    spl.putConstraint(SpringLayout.SOUTH,scrollbar,0,SpringLayout.SOUTH,colorboxpanel);
    spl.putConstraint(SpringLayout.NORTH,scrollbar,0,SpringLayout.NORTH,this.getContentPane());
    spl.putConstraint(SpringLayout.NORTH,colorboxpanel,0,SpringLayout.NORTH,this.getContentPane());

    this.getContentPane().add(colorboxpanel);
    this.getContentPane().add(scrollbar);
    
    colorboxpanel.addKeyListener(cellautoapp.keylis);
    scrollbar.addKeyListener(cellautoapp.keylis);
    colorboxpanel.setFocusable(true);
  }
}
class colorboxpanelmouselisc implements MouseListener{
  public void mouseClicked(MouseEvent e){}
  public void mouseEntered(MouseEvent e){}
  public void mouseExited(MouseEvent e){}
  public void mousePressed(MouseEvent e){
    int x = e.getX() / colorboxpanelc.sqsize;
    int y = e.getY() / colorboxpanelc.sqsize;
    int i2 = (((cellautoapp.colorbox.colorboxpanel.offset+y)*colorboxpanelc.numcolorsperrow)+x);
    if (i2 < cellautoapp.cellauto131.numcolors){
      //select the color
      cellautoapp.colorbox.colorboxpanel.selcolor = i2;
    }
    cellautoapp.colorlistindex = cellautoapp.colorbox.colorboxpanel.selcolor;
    cellautoapp.colorbox.colorboxpanel.repaint();
    cellautoapp.updatestatusbar();
  }
  public void mouseReleased(MouseEvent e){
    
  }
  
}
class colorboxpanelc extends JPanel{
  colorboxpanelmouselisc mouselis;
  int offset;
  static int selcolor;
  static int sqsize = 32;
  static int numcolorsperrow = 8;
  colorboxpanelc(){
    mouselis = new colorboxpanelmouselisc();
    addMouseListener(mouselis);
    selcolor = 0;
    offset = 0;
  }
  public Dimension getPreferredSize(){
    return new Dimension(colorboxpanelc.sqsize*numcolorsperrow,colorboxpanelc.sqsize*2);
  }
  
  //this function paints the color box
  protected void paintComponent(Graphics g){ 
     super.paintComponent(g);
     Rectangle r = this.getBounds();
     int w = r.width;
     int h = r.height;
     int numcolorrows = 2;
     
    //draw the colors using rectangles
    for (int y = 0;y < numcolorrows;y++){
      for (int x = 0;x < numcolorsperrow;x++){
        int i = (((offset+y)*numcolorsperrow)+x);
        if (i < cellautoapp.cellauto131.numcolors) {
          g.setColor(cellautoapp.cellauto131.getcellvaluecolor(i));
        }else{
          g.setColor(Color.gray);
        }
        g.fillRect(x*colorboxpanelc.sqsize,y*colorboxpanelc.sqsize,colorboxpanelc.sqsize,colorboxpanelc.sqsize);
        g.setColor(Color.black);
        if ((i > 0) & (i < cellautoapp.cellauto131.numcolors)) {
          g.drawString(cellautoapp.cellauto131.get_cellnotename(i-1),(x*sqsize)+6,(y*sqsize)+20);        
        }
      }
    }
    
    //draw a outline around the selected color
    int selcolorx = selcolor % numcolorsperrow;
    int selcolory = selcolor / numcolorsperrow;
    if ((selcolory-offset) >= 0){
      if ((selcolory-offset) < numcolorrows){
        g.setColor(new Color(255,0,0));
        for (int i = 0;i < 2;i++){
          g.drawRect((selcolorx*colorboxpanelc.sqsize)+i,((selcolory-offset)*colorboxpanelc.sqsize)+i,colorboxpanelc.sqsize-(i*2),colorboxpanelc.sqsize-(i*2));
        }
      }
    }
    
    //draw the grid 
    g.setColor(Color.blue);
    for (int x = 1;x < numcolorsperrow;x++){
      g.drawLine(x*sqsize,0,x*sqsize,numcolorrows*sqsize);
    }
    for (int y = 1;y < numcolorrows;y++){
      g.drawLine(0,y*sqsize,numcolorsperrow*sqsize,y*sqsize);
    }
  }  
}

class boardstate {
  boardstate(){
  }
  boardc board;
  cellautorulec rule;
  int generation;
  static int s;
}



class cellautoapp {
  static int PLAY_LIST_WINDOW = 0;
  static int RANDOM_TABLE_WINDOW = 1;
  
  static boolean auto_update_rule_table = true;
  static boolean edit_rows_mode = false;
  static boolean edit_notes_mode = false;
  static Random rand;
  static Random rand2;
  static newgamedialogc newgamedialog;
  static long lowest_long_value = 0L;
  static playlistwindowc playlistwindow;
  static int max_note_distance = 4;
  static int offsetx = 0;                             
  static int offsety = 0;
  static int movestep = 4;
  static colorboxc colorbox;
  static cellautomainwindowc cellautomainwindow = null;
  static JFrame cellautowindowframe = null;
  static JMenu filemenu;
  static JMenu editmenu;
  static JMenu viewmenu;
  static keylisc keylis;
  static actlisc actlis;
  static winlisc winlis;
  static mousemotionlisc mousemotionlis;
  static mouselisc mouselis;
  static String programname = "frank's cellular automaton music program";
  static String programversion = " v 1.2 ";
  static String programversion2 = "1.2";
  static String programversion3 = programversion2;
  static cellautorulec cellauto131;
  static edittablewindowc edittablewindow = null;
  static int colorlistindex = 0;
  static boardc board;
  static JLabel cellautostatusbar;
  static boolean gridon = true;
  static int gridminsize = 8;
  static boolean ZOOM_IN = true;
  static boolean ZOOM_OUT = false;
  static Stack undodata;
  static Stack redodata;
  static editdescriptionwindowc editdescriptionwindow;
  static optionsdialogc optionsdialog;
  static boolean keywordfound = false;
  static long board_song_seed = 1;
  static long rule_table_seed = 1;
  static long colored_note_seed = 1;
  static int tempo = 24;
  static int num_rows_to_play = 1;
  static int min_note_length = 24;
  static int empty_cell_space_bonus = 12;
  static int key_to_play_in = 5;
  static int melody_instrument = 2;
  static int chord_instrument = 49;
  static int chord_volume = 30;
  static int LOWER_OCTAVE = 0;
  static int UPPER_OCTAVE = 1;
  static int BOTH_OCTAVES = 2;
  static int play_octaves = LOWER_OCTAVE;
  static int num_notes_at_a_time = 1;
  static boolean playlistrunning = false;
  static note_playerc note_player;
  static CA_Playerc CA_Player;
  static randomize_notes_dialogc randomize_notes_dialog;
  static randomize_table_dialogc randomize_table_dialog;
  static randomtabledialogc randomtabledialog;
  //static optionsdialogc optionsdialog;
  cellautoapp() {
    
  }
  static void songisdone() {
    if (playlistrunning) { 
      int i = playlistwindow.listbox.getSelectedIndex();      
      if (i < (playlistwindow.list.size() - 1)) {
        playlistwindow.listbox.setSelectedIndex(i + 1);
        playlistwindow.playsong();
      } else {
        playlistrunning = false;
      }
    }
  }
  static void playnoteonkeyboard(int n) {
    int numcolors = cellautoapp.cellauto131.numcolors;
    cellautoapp.note_player.playnoteonkeyboard(n);
    int n2 = cellautorulec.cellnoteindex[n % 12] + 1;
    if (n2 < numcolors) {
      cellautoapp.colorlistindex = n2;
      cellautoapp.colorbox.colorboxpanel.selcolor = n2;
      cellautoapp.colorbox.colorboxpanel.repaint();
      cellautoapp.updatestatusbar();    
    }
  }
  static void move_notes_around(int x1,int y1,int x2,int y2) {
      if (y1 == y2) {
        int dx = x2 - x1;
        if (dx < 0) {
          int sq = cellautoapp.board.getsq(x1,y2);
          for (int i = 0;i > dx;i--) {
            int sq2 = cellautoapp.board.getsq(x1+i-1,y2);
            cellautoapp.board.setsq(x1+i,y2,sq2);
          }
          cellautoapp.board.setsq(x1+dx,y2,sq);
        }
        if (dx == 0) {
          if (auto_update_rule_table == true) {
            cellautoapp.board.setsq2(cellautoapp.cellauto131,x2,y2,cellautoapp.colorlistindex);
          } else {
            cellautoapp.board.setsq(x2,y2,cellautoapp.colorlistindex);
          }
        }
        if (dx > 0){
          int sq = cellautoapp.board.getsq(x1,y2);
          for (int i = 0;i < dx;i++) {
            int sq2 = cellautoapp.board.getsq(x1+i+1,y2);
            cellautoapp.board.setsq(x1+i,y2,sq2);
          }
          cellautoapp.board.setsq(x1+dx,y2,sq);
        }
      }
  }
  static void update_board_song_seed() {
    board_song_seed = board_song_seed + 1;
    if (optionsdialog != null) {
      if (optionsdialog.isVisible() == true) {
        optionsdialog.seededit1.setText(Long.toString(board_song_seed));
      }
    }
  }
  static void update_rule_table_seed() {
    rule_table_seed = rule_table_seed + 1;
    if (optionsdialog != null) {
      if (optionsdialog.isVisible() == true) {
        optionsdialog.seededit2.setText(Long.toString(rule_table_seed));
      }
    }
  }
  static void update_colored_note_seed() {
    colored_note_seed = colored_note_seed + 1;
    if (optionsdialog != null) {
      if (optionsdialog.isVisible() == true) {
        optionsdialog.seededit3.setText(Long.toString(colored_note_seed));
      }
    }
  }
  static void randomtable2() {
      cellautoapp.update_rule_table_seed();
      cellautoapp.rand2.setSeed(cellautoapp.rule_table_seed);
      cellautoapp.randomtable();
      if (cellautoapp.randomize_table_dialog == null) {
        cellautoapp.randomize_table_dialog = new randomize_table_dialogc("randomize table");
      }
      cellautoapp.randomize_table_dialog.seededit.setText(Long.toString(cellautoapp.rule_table_seed));
      if (cellautoapp.randomize_table_dialog.isVisible() == false) {
        cellautoapp.randomize_table_dialog.setVisible(true);
      }
      cellautoapp.edittablewindow.repaint();  
  }
  static void randomnotes2() {
      cellautoapp.update_colored_note_seed();
      cellautoapp.rand2.setSeed(cellautoapp.colored_note_seed);
      cellautoapp.randomnotes();
      cellautoapp.board.updateboard2(cellautoapp.cellauto131);
      if (cellautoapp.randomize_notes_dialog == null) {
        cellautoapp.randomize_notes_dialog = new randomize_notes_dialogc("randomize notes");
      }
      cellautoapp.randomize_notes_dialog.seededit.setText(Long.toString(cellautoapp.colored_note_seed));
      if (cellautoapp.randomize_notes_dialog.isVisible() == false) {
        cellautoapp.randomize_notes_dialog.setVisible(true);
      }
      cellautoapp.edittablewindow.repaint();
      cellautoapp.cellautomainwindow.repaint();
    
  }
  static void randomtable() {
      for (int x = 0;x < cellautoapp.cellauto131.tablewidth;x++) {
        for (int y = 0;y < cellautoapp.cellauto131.tableheight;y++) {
          int i = cellautoapp.rand2.nextInt(2);
          if (i == 1){
            int i2 = cellautoapp.colorlistindex;
            if (i2 == 0) {i2 = 1;}
            cellautoapp.cellauto131.setlookuptable(x,y,i2);
          } else {
            cellautoapp.cellauto131.setlookuptable(x,y,0);
          }
        }
      }
  }
  static int get_num_colors_in_rule_table() {
      cellautorulec c = cellautoapp.cellauto131;
      int nc = 0;
      for (int x = 0;x < c.tablewidth;x++) {
        for (int y = 0;y < c.tableheight;y++) {
          int sq = c.getlookuptable(x,y);
          if (sq > 0) {
            nc = nc + 1;
          }
        }
      }
      return nc;
  }
  static void randomnotes() {
      cellautorulec c = cellautoapp.cellauto131;
      int nc = get_num_colors_in_rule_table();
      int noteindex[] = new int[nc];
      for (int i = 0;i < nc;i++) {
        noteindex[i] = (i % (c.numcolors-1)) + 1;
      }
      for (int i = 0;i < nc;i++) {
        int j = cellautoapp.rand2.nextInt(i+1);
        int tmp = noteindex[i];
        noteindex[i] = noteindex[j];
        noteindex[j] = tmp;
      }

      nc = 0;
      for (int x = 0;x < c.tablewidth;x++) {
        for (int y = 0;y < c.tableheight;y++) {
          int sq = c.getlookuptable(x,y);
          if (sq > 0) {
            int i = noteindex[nc];
            nc = nc + 1;
            c.setlookuptable(x,y,i);
          }
        }
      }
  }
  static void open(String filename) {
    int l = filename.length();
    try{
      if (l > 0){
        String filetype = cellautoapp.getfiletype(filename);
        boolean r = opentext(new BufferedReader(new FileReader(filename)),filetype);
        if (r == false) {
             JOptionPane.showMessageDialog(cellautoapp.cellautowindowframe,new JLabel(
               "error: invalid file"),"error",JOptionPane.ERROR_MESSAGE);            
        }
      }
    } catch (Exception err){
      System.out.println("menuopen error");
      err.printStackTrace();
    }
    updatestatusbar();
    cellautoapp.colorbox.colorboxpanel.repaint();
    cellautoapp.edittablewindow.edittablepanel.repaint();
  }
  static void saveasxm(String filename) {
    saveasmidiorxm(filename,"xm");
  }

  static void saveasmidi(String filename) {
    saveasmidiorxm(filename,"midi");
  }
  static void saveasmidiorxm(String filename,String filetype2) {
    int l = filename.length();
    try {
      if (l > 0) {
        String filetype = cellautoapp.getfiletype(filename);

        if ((filetype.equals(filetype2) == false)) {
          filename = filename + "." + filetype2;
        }
        File f = new File(filename);
        if (f.exists() == false) {
          int min_note_length2 = cellautoapp.min_note_length;
          int empty_cell_space_bonus2 = cellautoapp.empty_cell_space_bonus;
          if (filetype2.equals("xm")) {
            cellautoapp.min_note_length = 2;
            cellautoapp.empty_cell_space_bonus = 1;
          }
          cellautoapp.CA_Player.clearnotelist();
          for (int i = 0;i < cellautoapp.board.height;i++){
            int i2 = (cellautoapp.board.generation+i+1) % cellautoapp.board.height;
            cellautoapp.CA_Player.play_row(cellautoapp.board,i2);
          }
          cellautoapp.CA_Player.play_last_note();
          notelistc l2 = cellautoapp.CA_Player.getnotelist();
          l2.addchordstonotes();
          if (filetype2.equals("xm")) {
            cellautoapp.note_player.output_notelist_to_xm_file(l2,f,cellautoapp.tempo);
          } else {
            cellautoapp.note_player.output_notelist_to_file(l2,f,cellautoapp.tempo);
          }
          cellautoapp.min_note_length = min_note_length2;
          cellautoapp.empty_cell_space_bonus = empty_cell_space_bonus2;
        } else {
          JOptionPane.showMessageDialog(cellautoapp.cellautowindowframe,new JLabel("file already exists"),"error",JOptionPane.ERROR_MESSAGE);          
        }
      }
    } catch (Exception err){
      System.out.println("menuopen error");
      err.printStackTrace();
    }
    updatestatusbar();
    cellautoapp.colorbox.colorboxpanel.repaint();
    cellautoapp.edittablewindow.edittablepanel.repaint();
  }
  static void save(String filename) {
    int l = filename.length();
    try{
      if (l > 0){
        String filetype = cellautoapp.getfiletype(filename);
        
        if ((filetype.equals("mca") == false)) {
          filename = filename + ".mca";
        }
        File f = new File(filename);
        if (f.exists() == false){
          savetext(new BufferedWriter(new FileWriter(filename,true)));
        } else {
          JOptionPane.showMessageDialog(cellautoapp.cellautowindowframe,new JLabel("file already exists"),"error",JOptionPane.ERROR_MESSAGE);          
        }
      }
    } catch (Exception err){
      System.out.println("menuopen error");
      err.printStackTrace();
    }
    updatestatusbar();
    cellautoapp.colorbox.colorboxpanel.repaint();
    cellautoapp.edittablewindow.edittablepanel.repaint();
  }
  static void savetext(BufferedWriter w){
    cellautorulec c = cellauto131;
    boardc b = board;
    double version = 0;
    try{
      PrintWriter p = new PrintWriter(w);
      p.println("CA_Music,4836");
      p.println("programname," + programname);
      p.println("programversion," + programversion3);
      p.println("offset," + cellautoapp.offsetx + "," + cellautoapp.offsety);
      p.println("zoom," + cellautomainwindowc.sqsize);
      p.println("numnotes," + (cellauto131.numcolors-1));
      p.println("board_seed," + board_song_seed);
      p.println("table_seed," + rule_table_seed);
      p.println("colored_note_seed," + colored_note_seed);
      p.println("tempo," + tempo);
      p.println("num_rows_to_play," + num_rows_to_play);
      p.println("min_note_length," + min_note_length);
      p.println("empty_cell_space_bonus," + empty_cell_space_bonus);
      p.println("key_to_play_in," + key_to_play_in);
      p.println("melody_instrument," + melody_instrument);
      p.println("chord_instrument," + chord_instrument);
      p.println("chord_volume," + chord_volume);
      p.println("num_notes_at_a_time," + num_notes_at_a_time);
      p.println("play_octaves," + play_octaves);
      p.println("select_chord_octave," + cachord.select_chord_octave);
      
      p.println("generation," + board.generation);
      if (c.description != null){
        String descline[] = c.description.split("\n");
        
        for (int i = 0;i < descline.length;i++){
          p.println("description," + descline[i]);
        }
      }

      //write the rule table
      p.println("cleartable," + c.neighborhoodsize);
      
      
      for (int y = 0;y < c.tableheight;y++){
        p.print("table");
        for (int x = 0;x < c.tablewidth;x++){
          int i = c.getlookuptable(x,y);
          p.print(","+i);
        }
        p.println();
      }
      //write the board
      p.println("clearboard,"+b.width+","+b.height);
      for (int y = 0;y < b.height;y++){
        if (b.emptyline(y) == false){
          String rle = b.writerle(y);
          p.println("line,"+y+","+rle);
        }
      }
      p.close();
    } catch (Exception e){
      e.printStackTrace();
    }
    
  }


  static boolean opentext(BufferedReader r,String filetype){
    cellautorulec c = cellauto131;
    boardc b = board;
    int bordery = 0;
    int borderflip = 0;
    int tabley = 0;
    int tableusedy = 0;
    int numcells = 0;
    int fgcolor = 0;
    int bgcolor = 0;
    int rselx = 0;
    int rsely = 0;
    String line = null;
    int linenum = 0;
    int fileerror = 0;
    double version2 = 0;
    double version = 0;
    boolean new_table64 = false;
    String ss[];
    String ss2[];
    
    Vector bgcolor_string_list = new Vector();
    c.description = null;

    try{
      version2 = Double.valueOf(programversion2).doubleValue();
      line = r.readLine();
      ss = line.split(",");
      if (ss.length < 2) {return false;}
      ss[0].trim();
      ss[1].trim();
      if (filetype.equals("mca") == false) {
        if ((ss[0].equals("CA_Music") == false)) {return false;}
        if ((ss[1].equals("4836") == false)) {return false;}
      }
      linenum = linenum + 1;
      while (line != null){
        ss = line.split(",");
        ss2 = new String[ss.length];
        for (int i = 0;i < ss.length;i++){
          ss2[i] = ss[i];
          ss[i] = ss[i].trim();
        }
        String st = ss[0];
        if (st.equals("programversion")){
          version = Double.valueOf(ss[1]).doubleValue();
          if (version > version2){
              JOptionPane.showMessageDialog(cellautoapp.cellautowindowframe,new JLabel(
                "can not load file, it is too new.  the program version is " + programversion2 + 
                ", the file version is " + version),"error",JOptionPane.ERROR_MESSAGE);
              break;
          }
        }

        if (st.equals("board_seed")) {
          board_song_seed = Long.valueOf(ss[1]).longValue();
        }
        if (st.equals("table_seed")) {
          rule_table_seed = Long.valueOf(ss[1]).longValue();
        }
        if (st.equals("colored_note_seed")) {
          colored_note_seed = Long.valueOf(ss[1]).longValue();
        }        
        if (st.equals("tempo")) {
          tempo = Integer.valueOf(ss[1]).intValue();
        }
        if (st.equals("num_rows_to_play")) {
          num_rows_to_play = Integer.valueOf(ss[1]).intValue();
        }
        if (st.equals("min_note_length")) {
          min_note_length = Integer.valueOf(ss[1]).intValue();
        }
        if (st.equals("empty_cell_space_bonus")) {
          empty_cell_space_bonus = Integer.valueOf(ss[1]).intValue();
        }
        if (st.equals("key_to_play_in")) {
          key_to_play_in = Integer.valueOf(ss[1]).intValue();
        }
        if (st.equals("melody_instrument")) {
          melody_instrument = Integer.valueOf(ss[1]).intValue();
        }
        if (st.equals("chord_instrument")) {
          chord_instrument = Integer.valueOf(ss[1]).intValue();
        }
        if (st.equals("chord_volume")) {
          chord_volume = Integer.valueOf(ss[1]).intValue();
        }
        if (st.equals("num_notes_at_a_time")) {
          num_notes_at_a_time = Integer.valueOf(ss[1]).intValue();
        }
        if (st.equals("play_octaves")) {
          play_octaves = Integer.valueOf(ss[1]).intValue();
        }
        if (st.equals("select_chord_octave")) {
          cachord.select_chord_octave = Integer.valueOf(ss[1]).intValue();
        }

        if (st.equals("seed")){
          c.seed = Integer.valueOf(ss[1]).intValue();
        }
        if (st.equals("generation")){
          board.generation = Integer.valueOf(ss[1]).intValue();
        }
        if (st.equals("line")){
          int r2 = Integer.valueOf(ss[1]).intValue();
          String rle = ss[2];
          b.readrle(r2,rle);
        }
        if (st.equals("numnotes")){
          int i = Integer.valueOf(ss[1]).intValue();
          c.numcolors = i + 1;
          //c.setnumcolors(i);
        }
        if (st.equals("clearboard")){
          if (newgamedialog != null){
            newgamedialog.widthedit.setText(ss[1]);
            newgamedialog.heightedit.setText(ss[2]);
          }
          int x = Integer.valueOf(ss[1]).intValue();
          int y = Integer.valueOf(ss[2]).intValue();
          b = new boardc(x,y);
          board = b;
        }
        if (st.equals("cleartable")) {
          int i = Integer.valueOf(ss[1]).intValue();
          c.cleartable(i);
        }
        if (st.equals("table")) {
          //read the rule table
          int i = 1;
          int tablex = 0;
          while (i < ss.length){
            int i2 = Integer.valueOf(ss[i]).intValue();
            c.setlookuptable(tablex,tabley,i2);
            tablex = tablex + 1;
            i++;
          }
          tablex = 0;
          tabley = tabley + 1;
        }
        
        if (st.equals("description")){
          //read the description
          int i = 1;
          while (i < ss.length){
            if (c.description != null){
              c.description = c.description + ss[i];
            }else{
              c.description = ss[i];
            }
            if (c.description != null){
              if (i < (ss.length-1)){
                c.description = c.description + ",";
              }else{
                c.description = c.description + "\n";
              }
            }else{
              if (i < (ss.length-1)){
                c.description = ",";
              }else{
                c.description = "\n";
              }
            }
            i++;
          }
        }
        line = r.readLine();
        linenum = linenum + 1;
      }
      cellautoapp.colorbox.colorboxpanel.repaint();
      if (cellautoapp.editdescriptionwindow != null) {
        cellautoapp.editdescriptionwindow.text.setText(cellautoapp.cellauto131.description);
      }
      r.close();
    } catch (IOException e){
      e.printStackTrace();
      return false;
    } catch (NumberFormatException e) {
      System.out.println("error: can not read number, line number is " + linenum + " line is ");
      System.out.println(line);
      fileerror = 1;
    }
    if (fileerror == 1){
      JOptionPane.showMessageDialog(cellautoapp.cellautowindowframe,new JLabel("can not open the file, " + 
        "see the console for details"),"error",JOptionPane.ERROR_MESSAGE);
      return false;
    }
    return true;
  }      
  static void updatestatusbar() {
    int n = (undodata.size() + boardstate.s);
    int n2 = redodata.size();

    String str = "generation: " + board.generation + " undo: " + n;    
    str = str + " redo: " + n2;
    str = str + " mode: ";
    if (edit_rows_mode == true) {
      str = str + "r ";
    }
    if (edit_notes_mode == true) {
      str = str + "e ";
    }
    str = str + " key: " + cellautoapp.cellauto131.get_cellnotename(0);
    cellautostatusbar.setText(str);
  }
  static String getfiletype(String filename){
    String type = "";
    for (int i = filename.length()-1;i >= 0;i--){
      char ch = filename.charAt(i);
      if (ch == '.'){break;}                       
      if (ch == '\\'){break;}
      if (ch == '/'){break;}
      type = ch + type;
    }
    return type;
  }  
  //this function zooms in or out
  static void zoom(int x,int y,boolean zoomtype){
    int w = cellautomainwindow.getWidth();
    int h = cellautomainwindow.getHeight();
    if (zoomtype == ZOOM_IN){
      cellautomainwindowc.sqsize = cellautomainwindowc.sqsize + 1;
    }
    if (zoomtype == ZOOM_OUT){
      if (cellautomainwindowc.sqsize > 1){
        cellautomainwindowc.sqsize = cellautomainwindowc.sqsize - 1;
      }
    }
    offsetx = x - (w/(cellautomainwindowc.sqsize*2));
    offsety = y - (h/(cellautomainwindowc.sqsize*2));
    cellautoapp.cellautomainwindow.repaint();
  }

  public static String getkey(String name){
    keywordfound = true;
    return name.toUpperCase();
  }
  public static char getkey2(String name){
    keywordfound = true;
    if (name.equals("space")){return ' ';}
    if (name.equals("comma")){return ',';}
    if (name.equals("dot")){return '.';}
    if (name.equals("period")){return '.';}
    if (name.equals("nokey")){return 0;}

    return Character.toLowerCase(name.charAt(0));
  }
    
  //read the options from a file
  public static void readoptionsfromfile(String filename){
    File f = new File(filename);
    String line = null;
    int linenum = 1;
    String errorstr = null;
    boolean iserror = false;
    boolean duplicate_keyword = false;
    if (f.exists()){
      System.out.println("reading " + filename);
      try{
        HashMap hash = new HashMap();
        BufferedReader r2 = new BufferedReader(new FileReader(filename));
        line = r2.readLine();
        while (line != null){
          keywordfound = false;
          duplicate_keyword = false;
          String ss[] = line.split(",");
          for (int i = 0;i < ss.length;i++){
            ss[i] = ss[i].trim();
          }
          String st = ss[0];
          String ss2[] = st.split("#");
          st = ss2[0];
          st = st.toLowerCase();
          if (st.equals("")){
            keywordfound = true;
          }
          if (st.length() > 0){
            if (hash.containsKey(st)){
              Integer i = (Integer) hash.get(st);
              errorstr = "error at line " + linenum + ": " + st + " is already defined at line " + i.intValue();
              System.out.println(errorstr);
              iserror = true;
            }
            if (duplicate_keyword == false) {
              hash.put(st,new Integer(linenum));
            }
          }
          if (st.equals("open_ctrl_key")) {
            cellautokeys.open_ctrl_key = getkey(ss[1]);
          }
          if (st.equals("save_ctrl_key")) {
            cellautokeys.save_ctrl_key = getkey(ss[1]);
          }
          if (st.equals("options_ctrl_key")) {
            cellautokeys.options_ctrl_key = getkey(ss[1]);
          }
          if (st.equals("random_table_ctrl_key")) {
            cellautokeys.random_table_ctrl_key = getkey(ss[1]);
          }
          if (st.equals("random_notes_ctrl_key")) {
            cellautokeys.random_notes_ctrl_key = getkey(ss[1]);
          }
          if (st.equals("prev_color_key")) {
            cellautokeys.prev_color_key = getkey2(ss[1]);
          }
          if (st.equals("next_color_key")) {
            cellautokeys.next_color_key = getkey2(ss[1]);
          }
          if (st.equals("play_row_key")) {
            cellautokeys.play_row_key = getkey2(ss[1]);
          }
          if (st.equals("save_to_stack_key")) {
            cellautokeys.save_to_stack_key = getkey2(ss[1]);
          }
          if (st.equals("undo_key")) {
            cellautokeys.undo_key = getkey2(ss[1]);
          }
          if (st.equals("redo_key")) {
            cellautokeys.redo_key = getkey2(ss[1]);
          }
          if (st.equals("edit_rows_mode_key")) {
            cellautokeys.edit_rows_mode_key = getkey2(ss[1]);
          }
          if (st.equals("edit_notes_mode_key")) {
            cellautokeys.edit_notes_mode_key = getkey2(ss[1]);
          }
          if (st.equals("no_note_key")) {
            cellautokeys.no_note_key = getkey2(ss[1]);
          }
          if (st.equals("C_key")) {
            cellautokeys.C_key = getkey2(ss[1]);
          }
          if (st.equals("CS_key")) {
            cellautokeys.CS_key = getkey2(ss[1]);
          }
          if (st.equals("D_key")) {
            cellautokeys.D_key = getkey2(ss[1]);
          }
          if (st.equals("DS_key")) {
            cellautokeys.DS_key = getkey2(ss[1]);
          }
          if (st.equals("E_key")) {
            cellautokeys.E_key = getkey2(ss[1]);
          }
          if (st.equals("F_key")) {
            cellautokeys.F_key = getkey2(ss[1]);
          }
          if (st.equals("FS_key")) {
            cellautokeys.FS_key = getkey2(ss[1]);
          }
          if (st.equals("G_key")) {
            cellautokeys.G_key = getkey2(ss[1]);
          }
          if (st.equals("GS_key")) {
            cellautokeys.GS_key = getkey2(ss[1]);
          }
          if (st.equals("A_key")) {
            cellautokeys.A_key = getkey2(ss[1]);
          }
          if (st.equals("AS_key")) {
            cellautokeys.AS_key = getkey2(ss[1]);
          }
          if (st.equals("B_key")) {
            cellautokeys.B_key = getkey2(ss[1]);
          }
          if (st.equals("gridminsqsize")){
            cellautoapp.gridminsize = Integer.valueOf(ss[1]).intValue();
            keywordfound = true;
          }     
          line = r2.readLine();
          linenum = linenum + 1;
        }
        r2.close();
      } catch (IOException e){
        e.printStackTrace();
      } catch (NumberFormatException e) {
        errorstr = "error at line " + linenum + ": can not read number";
        System.out.println("error: can not read number, line number is " + linenum + " line is ");
        System.out.println(line);
        iserror = true;
      }
      if (iserror == true){
        JOptionPane.showMessageDialog(cellautoapp.cellautowindowframe,new JLabel(errorstr),"error",JOptionPane.ERROR_MESSAGE);  
      }
    }
  }

  //this function saves the board state to the undo stack or the redo stack
  static void calc(int n) {
    cellautoapp.redodata.clear();
    if (boardstate.s == 1){
      cellautoapp.saveboard(cellautoapp.undodata);
    }
    for (int i = 0;i < n;i++) {
      cellautoapp.cellauto131.calc(cellautoapp.board);
    }
  }
  static void saveboard(Stack s) {
    //System.out.println("saveboard");
    boardstate b = new boardstate();
    
    b.board = new boardc(board);
    b.rule = new cellautorulec(cellauto131);    
    b.generation = board.generation;
    boardstate.s = 0;
    s.push(b);
  }
  static void restoreboard(Stack s) {
    if (s.empty() == true) {return;}
    //System.out.println("restoreboard");
    boardstate b = (boardstate) s.pop();
    boardstate.s = 1;
    cellauto131 = b.rule;
    board = b.board;
    board.generation = b.generation;
    cellautoapp.updatestatusbar();
    
  }
  public static void main(String[] args){
      javax.swing.SwingUtilities.invokeLater(new Runnable() {
        public void run() {
          rule_table_seed = System.currentTimeMillis();
          board_song_seed = rule_table_seed-1;
          colored_note_seed = rule_table_seed;
          //lowest_long_value = (1 << 63) | (1 << 62) | ((1 << 62) - 1);
          lowest_long_value = (1L << 63);
          //System.out.println(lowest_long_value);
          readoptionsfromfile("options.txt");
          undodata = new Stack();
          redodata = new Stack();
          rand = new Random();
          rand2 = new Random();
          note_player = new note_playerc();
          CA_Player = new CA_Playerc();
          
          board = new boardc(100,100);
          cellauto131 = new cellautorulec();
          mouselis = new mouselisc();
          mousemotionlis = new mousemotionlisc();
          keylis = new keylisc();
          actlis = new actlisc();
          winlis = new winlisc();
          JMenuBar mb = new JMenuBar();
          filemenu = new JMenu("file");
          mb.add(filemenu);
          
          JMenuItem minew = new JMenuItem("new");

          filemenu.add(minew);
          minew.setActionCommand("menunew");    
          minew.addActionListener(cellautoapp.actlis);
          
          
          JMenuItem miopen = new JMenuItem("open ctrl-" + cellautokeys.open_ctrl_key.toLowerCase());
          filemenu.add(miopen);
          miopen.setActionCommand("menuopen");
          miopen.addActionListener(cellautoapp.actlis);
          
          JMenuItem misave = new JMenuItem("save ctrl-" + cellautokeys.save_ctrl_key.toLowerCase());
          filemenu.add(misave);
          misave.setActionCommand("menusave");
          misave.addActionListener(cellautoapp.actlis);

          JMenuItem misaveasmidi = new JMenuItem("save as midi");
          filemenu.add(misaveasmidi);
          misaveasmidi.setActionCommand("menusaveasmidi");
          misaveasmidi.addActionListener(cellautoapp.actlis);

          JMenuItem misaveasxm = new JMenuItem("save as xm");
          filemenu.add(misaveasxm);
          misaveasxm.setActionCommand("menusaveasxm");
          misaveasxm.addActionListener(cellautoapp.actlis);
          
          JMenuItem mioptions = new JMenuItem("options ctrl-" + cellautokeys.options_ctrl_key.toLowerCase());
          filemenu.add(mioptions);
          mioptions.setActionCommand("menuoptions");
          mioptions.addActionListener(cellautoapp.actlis);
          
          JMenuItem midescription = new JMenuItem("view description");
          filemenu.add(midescription);
          midescription.setActionCommand("menudescription");
          midescription.addActionListener(cellautoapp.actlis);
          
          JMenuItem miexit = new JMenuItem("exit");
          filemenu.add(miexit);
          miexit.setActionCommand("menuexit");
          miexit.addActionListener(cellautoapp.actlis);

          //setup the edit menu
          editmenu = new JMenu("edit");
          JMenuItem mirandomtable = new JMenuItem("randomize table ctrl-" + cellautokeys.random_table_ctrl_key.toLowerCase());
          editmenu.add(mirandomtable);
          mirandomtable.setActionCommand("menurandomtable");
          mirandomtable.addActionListener(cellautoapp.actlis);

          JMenuItem mirandomnotes = new JMenuItem("randomize notes ctrl-" + cellautokeys.random_notes_ctrl_key.toLowerCase());
          editmenu.add(mirandomnotes);
          mirandomnotes.setActionCommand("menurandomnotes");
          mirandomnotes.addActionListener(cellautoapp.actlis);
          
          JMenuItem miaddnote = new JMenuItem("add note");
          editmenu.add(miaddnote);
          miaddnote.setActionCommand("menuaddnote");
          miaddnote.addActionListener(cellautoapp.actlis);

          JMenuItem micleartable = new JMenuItem("clear table");
          editmenu.add(micleartable);
          micleartable.setActionCommand("menucleartable");
          micleartable.addActionListener(cellautoapp.actlis);

          JMenuItem midoubletable = new JMenuItem("double table size");
          editmenu.add(midoubletable);
          midoubletable.setActionCommand("menudoubletable");
          midoubletable.addActionListener(cellautoapp.actlis);
          
          mb.add(editmenu);
          
          //setup the view menu
          viewmenu = new JMenu("view");
          
          JMenuItem miedittable = new JMenuItem("show rule table");
          viewmenu.add(miedittable);
          miedittable.setActionCommand("menushowedittablewindow");
          miedittable.addActionListener(cellautoapp.actlis);
          
          JMenuItem micolorbox = new JMenuItem("show color box");
          viewmenu.add(micolorbox);
          micolorbox.setActionCommand("menushowcolorbox");
          micolorbox.addActionListener(cellautoapp.actlis);
          mb.add(viewmenu);

          JMenuItem mirandomtabledialog = new JMenuItem("show random table dialog");
          viewmenu.add(mirandomtabledialog);
          mirandomtabledialog.setActionCommand("showrandomtabledialog");
          mirandomtabledialog.addActionListener(cellautoapp.actlis);


          JMenuItem mishowplaylist = new JMenuItem("show play list");
          viewmenu.add(mishowplaylist);
          mishowplaylist.setActionCommand("showplaylist");
          mishowplaylist.addActionListener(cellautoapp.actlis);
          
          
          cellautowindowframe = new JFrame(cellautoapp.programname + " " +  cellautoapp.programversion);
          cellautomainwindow = new cellautomainwindowc();
          colorbox = new colorboxc("color box");
          colorbox.setBounds(0,0,(colorboxpanelc.sqsize*8)+30,(colorboxpanelc.sqsize*2)+40);
          colorbox.setVisible(true);
          edittablewindow = new edittablewindowc("rule table window");
          edittablewindow.setBounds(300,0,300,100);
          edittablewindow.setVisible(true);
          
          cellautomainwindow.addMouseListener(mouselis);
          cellautomainwindow.addMouseMotionListener(mousemotionlis);
          cellautowindowframe.setJMenuBar(mb);
          SpringLayout spl = new SpringLayout();
          cellautowindowframe.getContentPane().setLayout(spl);
          spl.putConstraint(SpringLayout.EAST,cellautomainwindow,0,SpringLayout.EAST,cellautowindowframe.getContentPane());
          spl.putConstraint(SpringLayout.WEST,cellautomainwindow,0,SpringLayout.WEST,cellautowindowframe.getContentPane());
          
          cellautostatusbar = new JLabel("status bar"); 
          spl.putConstraint(SpringLayout.SOUTH,cellautostatusbar,0,SpringLayout.SOUTH,cellautowindowframe.getContentPane());
          spl.putConstraint(SpringLayout.SOUTH,cellautomainwindow,0,SpringLayout.NORTH,cellautostatusbar);
          spl.putConstraint(SpringLayout.NORTH,cellautomainwindow,0,SpringLayout.NORTH,cellautowindowframe.getContentPane());
          cellautowindowframe.getContentPane().add(cellautostatusbar);
          cellautowindowframe.getContentPane().add(cellautomainwindow);
          cellautowindowframe.pack();
          cellautowindowframe.addWindowListener(cellautoapp.winlis);
          updatestatusbar();
          cellautowindowframe.repaint();
          cellautowindowframe.setBounds(0,100,624,435);
          cellautowindowframe.setVisible(true);
          cellautowindowframe.setFocusable(true);
          cellautowindowframe.addKeyListener(keylis);
          cellautomainwindow.setFocusable(true);
          newgamedialog = new newgamedialogc("new song");
          newgamedialog.newgame();
        }
      });    
    
  }

}

